      include "readall.f"
C
C     Real*4 runwrap
C
      SUBROUTINE runwrap(xrf,xunrf)  ! customized
C
      implicit none
      include "params.h"
      include "dimensions.h"
      include "commons.h"
C
C     *****local variables
C
      integer iat,jat,ibond,kk
      real*4 xrf(3,maxat),xunrf(3,maxat),diff
      real*4 rboxgen,el2gen
      common /msd1/ rboxgen,el2gen
C
      do iat = 1,nat
        xunrf(1,iat) = xrf(1,iat)
        xunrf(2,iat) = xrf(2,iat)
        xunrf(3,iat) = xrf(3,iat)
      end do
C
C     *****unwrap by bonds
C
      do ibond = 1,nbonds
        iat = bonds(1,ibond)
        jat = bonds(2,ibond)
        do kk = 1,3
          diff = xrf(kk,jat) - xrf(kk,iat)
          if (abs(diff) .gt. el2gen)
     $      diff = diff - sign(rboxgen,diff)
          xunrf(kk,jat) = xunrf(kk,iat) + diff
        end do
      end do
      return
      end
C
C     ***  ACF program *********
C
C     version v2.0 written by C. J. F. Solano
C
      PROGRAM ACF
      implicit none
C
      include "params.h"
      include "dimensions.h"
      include "commons.h"
C
      integer maxshots
      parameter (maxshots=30101) ! maximum number of snaps
      real*8 xcm(3,maxshots,maxat)
      real*4 xx(3,maxat),xold(3,maxat),aa,xun(3,maxat)
      real*4 xreal(3,maxat),xrealold(3,maxat)
      real*4 rboxgen,el2gen
      real*4 time
      integer numbch(maxat,2)
      real*8, allocatable :: ech(:)
      integer, allocatable :: ifn(:),iln(:),numberp(:),natchtp(:) 
      real*8 echain(maxnch),dispmax
      real*8 qtmp,tmp,chainmass,ttime
      integer ntot(2),norigin,numchtp,atmtyp
      integer nfiles
      integer ichain,iat,itype,itmp,i,jj,ipt
      integer ifiles,natoms,isnap,j,ipos,ich,nshots,
     $  npos,ik,itek,iii
      character*50, allocatable :: fname(:)
      logical ok,logaut
      real*8 toldist2,dist02,dist2,ACFF
      integer hij0,hij,val0,val0old,val
      common /msd1/rboxgen,el2gen
C
C     I) Initializations
C
C     Files for analysis
      call readinit()
      open(22,file=fname1,status="old")
      read(22,*) nfiles
      if (nfiles.lt.1) then
        write(6,*) 'nfiles < 1'
        stop
      endif
      allocate (fname(nfiles))
      do i=1,nfiles
        read(22,'(a50)') fname(i)
        write(6,'(a50)') fname(i)
        open (30+i,file=fname(i),form='unformatted',status='old')
      end do
      write(6,*) 'Opened ',nfiles,' fort.77 (Instantaneous' 
     $         //' Coordinates) files'
      close(22)
C        
      call read12()
      call read11()
      call read26()
C
C    II) Determine possible molec types, molec charges, 
C        number of molecs of each type, and atom types
C
      if (Qmolec) then
        do ichain=1,maxnch
          echain(ichain)=0.0d0
        end do
        do iat=1,nat
          echain(chain(iat))=echain(chain(iat))+q(atomtype(iat))
        end do  ! next iat
C
        allocate (ech(nptypes),ifn(nptypes),iln(nptypes),
     $            numberp(nptypes),natchtp(nptypes))
        ! chain type '1'
        ifn(1)=1
        itype=1
        itmp=natch(1)
        qtmp=echain(1)
        ichain=1
        do while (ichain.lt.maxnch)
          ichain=ichain+1
          if (natch(ichain).ne.itmp.or.
     $        abs(echain(ichain)-qtmp).gt.0.01) then
            ! chain type 'itype'
            iln(itype)=ichain-1
            ! chain type 'itype+1'
            itype=itype+1
            if (itype.gt.nptypes) then
              write(6,*) 'Number of molec types is greather than ',
     $                   nptypes
              stop
            endif
            ifn(itype)=ichain
            itmp=natch(ichain)
            qtmp=echain(ichain)
          endif
        end do  ! next ichain
        ! chain type 'itype'
        iln(itype)=maxnch
        if (itype.ne.nptypes) then
          write(6,*) 'In automatically assignment there are ',itype,
     $               ' molec types'
          stop  
        endif
        do i=1,nptypes
          write(6,*) labeltyp(i),' type for molecs from ',ifn(i),
     $               ' to ',iln(i)     
        end do
C
        do i=1,nptypes
C         Determine the charge and number of atoms for the first chain of this type
          ech(i)=0.0d0
          natchtp(i)=0
          numchtp=0
          do iat=1,nat
            if (chain(iat).eq.ifn(i)) then  ! choose the first chain of the type
              ech(i)=ech(i)+q(atomtype(iat))
              natchtp(i)=natchtp(i)+1
              if (.not.ldummy(iat)) numchtp=numchtp+1
            endif
          end do   ! next iat
C         Check to make sure that all chain of the same type have the same charge
          do jj=ifn(i),iln(i)
            if (abs(echain(jj)-ech(i)).gt.1d-5)  then
              write(6,*) 'For the molec ',jj,' charge = ',echain(jj)
              write(6,*) 'but for the molec type ',i,' it is = ',
     $                    ech(i)
              stop
            endif 
          end do ! next jj
          write(6,*) 'For the molec type ',labeltyp(i),
     $               ' the charge is ',ech(i)
          write(6,*) 'For the molec type ',labeltyp(i),
     $               ' the number of atoms is',numchtp
        end do ! next i
C       Determine number of chains of each type and total number of chains
        i=0
        do ipt=1,nptypes
          numberp(ipt)=iln(ipt)-ifn(ipt)+1
          i=i+numberp(ipt)
          write(6,*) 'Type ',labeltyp(ipt),' has ',numberp(ipt),
     $               ' molecules'
        end do
        write(6,*) 'There are ',i,' molecules'
      else
        atmtyp=0
        do iat=1,nat
          if (atomtype(iat).gt.atmtyp) atmtyp=atomtype(iat)
        end do
        if (atmtyp.ne.maxcharges) then
          write(6,*) 'In automatically assignment there are ',atmtyp,
     $               ' atom types'
          stop
        end if
      end if
C
C     III) Determine molecs or atoms for (A)CF
C
      logaut=labels1.eq.labels2
      write(6,*) 'Lifetime correlation function for ',labels1,labels2
      j=0      
      jj=0
      if (Qmolec) then
        write(6,*) 'molec---new assignment'
        do ichain=1,maxnch
C         Obtention of molec type for 'ichain'
          i=0
          ok=.true.
          do while (i.lt.nptypes .and. ok)
            i=i+1
            if (abs(echain(ichain)-ech(i)).lt.1d-5 .and. 
     $          natch(ichain).eq.natchtp(i)) ok=.false.
          end do
          if (ok) then
            write(6,*) 'Error in the assignment of molecs for each type'
            stop
          end if
C         Select molec types for (A)CF
          if (labeltyp(i).eq.labels1) then  
            j=j+1
            numbch(j,1)=ichain
            write(6,*) ichain,j
          end if
          if (.not.logaut .and. labeltyp(i).eq.labels2) then
            jj=jj+1
            numbch(jj,2)=ichain
            write(6,*) ichain,jj
          end if
        end do
      else
        write(6,*) 'atom---new assignment'
        do iat=1,nat
C         Obtention of atom type for 'iat'
          i=0
          ok=.true.
          do while (i.lt.atmtyp .and. ok)
            i=i+1
            if (i.eq.atomtype(iat)) ok=.false.
          end do
          if (ok) then
            write(6,*) 'Error in the assignment of atoms for each type'
            stop
          end if  
C         Select atom types for (A)CF
          if (atom_labels(i).eq.labels1) then
            j=j+1
            numbch(j,1)=iat
            write(6,*) iat,j
          end if
          if (.not.logaut .and. atom_labels(i).eq.labels2) then
            jj=jj+1
            numbch(jj,2)=iat
            write(6,*) iat,jj
          end if
        end do
      end if
      ntot(1)=j
      ntot(2)=jj
C
C     IV) Obtention of molecular (CM) or atomic coordinates
C
      dispmax=0.0d0 ! maximum displacement
      ipos=0 ! counter for positions along time
      iii=0 ! counter for two-lines block in fort.77 files
      do ifiles = 31,30+nfiles
        ok = .true.
        do while (ok)
C         Read data from fort.77 file
          read(ifiles,iostat=i) natoms,time,isnap,box ! first line 
          if (i.eq.0) then
            if (natoms.ne.nat) then
              write(6,*) 'Wrong number of atoms in fort.77 file'
              write(6,*) natoms,' atoms is different to ',nat
              stop
            endif
            if (iii.eq.0) boxold=box
            if (box.ne.boxold) then
              write(6,*) 'Different box sizes : ',box,boxold
              stop
            end if
            read(ifiles,iostat=i) xx ! second line
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          endif
          if (i.eq.0) then
            ok = .true.
            iii=iii+1
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          else
            ok = .false. ! EOF
          endif
          if (ok .and. iii.gt.nskip) then
C           Periodic cubic cell
            rboxgen=real(box)
            el2gen=0.5*rboxgen
C           Check PBC
            do iat=1,nat
              do j=1,3
                if (xx(j,iat).lt.0.0 .or. xx(j,iat).gt.rboxgen) then
                  write(6,*) 'NO PBC for snapshot : ',isnap
                  write(6,*) 'chain---atom---(x=1,y=2,z=3)--- val'
                  write(6,*) chain(iat),
     &             atom_labels(atomtype(iat))(1:3),j,xx(j,iat)         
                  stop
                end if
              end do
            end do 
C           Treatment of the Cartesian coordinates
            ipos=ipos+1
            if (ipos.eq.1) then
              do iat=1,nat
                do j=1,3
                  xreal(j,iat)=xx(j,iat)
                end do
              end do
            else
              do iat=1,nat
                tmp=0.0d0
                do j=1,3
                  aa=xx(j,iat)-xold(j,iat)
                  if (abs(aa).gt.el2gen)
     $              aa=aa-sign(rboxgen,aa)
                  tmp=tmp+aa*aa
                  xreal(j,iat)=xrealold(j,iat)+aa
                end do  ! next j  
                if (tmp.gt.dispmax) dispmax=tmp
              end do    ! next iat
            endif       ! (ipos.eq.1)
            do iat=1,nat
              do j=1,3
                xold(j,iat)=xx(j,iat)
                xrealold(j,iat)=xreal(j,iat)
              end do
            end do
            call runwrap(xreal,xun) 
C           Obtention of center of mass for each molec or
C           cartesian coordinates for each atom
            if (mod(ipos,nread).eq.0) then
              nshots=ipos/nread
              if (maxshots.lt.nshots) then
                write(6,*) 'Increase the maxshots'
                write(6,*) 'Current maxshots = ',maxshots
                stop
              endif
              if (Qmolec) then
                do ich=1,maxnch
                  do j=1,3
                    xcm(j,nshots,ich)=0.0d0 ! time-dependent CM coordinates 
                  end do
                  chainmass=0.0d0
                  do iat=1,nat
                    if (chain(iat).eq.ich) then
                      chainmass=chainmass+mass(iat)
                      do j=1,3
                        xcm(j,nshots,ich)=xcm(j,nshots,ich)+
     $                                    xun(j,iat)*mass(iat)
                      end do
                    endif
                  end do
                  do j=1,3
                    xcm(j,nshots,ich)=xcm(j,nshots,ich)/chainmass
                  end do
                end do ! next ich
              else
                do iat=1,nat
                  do j=1,3
                    xcm(j,nshots,iat)=dble(xreal(j,iat))
                  end do
                end do
              end if 
            endif ! (mod(ipos,nread).eq.0)
          endif ! (ok .and. iii.ge.nskip)
        end do
        write(6,*) 'fort.77 file ',ifiles,':'
        write(6,*) natoms,' atoms, ',time,' time step (fs), ',box,
     $             ' cell size (Angs.)' 
        write(6,*) isnap,' final snapshot file, ',ipos,
     $             ' actual snapshot'
        close(ifiles)
      end do ! next ifiles
      if (nshots.lt.2) then
        write(6,*) 'nshots < 2'
        stop
      endif
      write(6,*) 'Maximum displacement (Angs.)',dsqrt(dispmax)
C
C     V) CF calculation       
C
C     Obtention of time origins
      norigin=nshots/2
      if (initval.gt.norigin) then
        write(6,*) 'Initial elapsed time value is greather than'
     $           //' maximum value of ',norigin
        stop
      endif
      write (6,*) 'There are ',nshots,' times'
      write (6,*) 'There are ',norigin,' time origins'
C     Open outputfile
      open (88,file=fname2,status='new')
      if (logaut) then
        write(88,'(a)') '# *************************************'
        write(88,'(a)') '# * Lifetime autocorrelation function *'
        write(88,'(a)') '# *************************************'
      else
        write(88,'(a)') '# *********************************'
        write(88,'(a)') '# * Lifetime correlation function *'
        write(88,'(a)') '# *********************************' 
      end if      
      do i=1,nfiles
        write(88,'("# fort.77 file = ",a50)') fname(i)
      end do
      write(88,'(a,i10,a,10i6)') "# v2.1:nfiles ",nfiles
      if (logaut) then
        write(88,'(a,1x,a5)') '# Lifetime ACF for :',labels1
        write(88,'(a)') "# elapsed time [ps] --- ACF --- log(ACF)"
      else 
         write(88,'(a,1x,a5,a1,a5)') '# Lifetime CF for :',labels1,'-',
     $                               labels2
         write(88,'(a)') "# elapsed time [ps] --- CF --- log(CF)"
      end if
C
      toldist2=toldist*toldist
      do npos=initval,norigin ! npos is the elapsed time
        val0=0 
        val=0
        do ik=1,norigin !ik is the starting position
          itek=ik+npos ! itek is the final position
          if (logaut) then
            do i=1,ntot(1)-1
              ichain=numbch(i,1)
              do j=i+1,ntot(1)
                ich=numbch(j,1)
                dist02=0.0d0
                dist2=0.0d0
                do jj=1,3
                  tmp=xcm(jj,ik,ichain)-xcm(jj,ik,ich)
                  dist02=dist02+tmp*tmp
                  tmp=xcm(jj,itek,ichain)-xcm(jj,itek,ich) 
                  dist2=dist2+tmp*tmp
                end do
                hij0=0
                if (dist02.le.toldist2) hij0=1
                hij=0
                if (dist2.le.toldist2) hij=1
                val0=val0+hij0
                val=val+hij0*hij
              end do ! next j 
            end do ! next i 
          else 
            do i=1,ntot(1)
              ichain=numbch(i,1)
              do j=1,ntot(2)
                ich=numbch(j,2)
                dist02=0.0d0
                dist2=0.0d0
                do jj=1,3
                  tmp=xcm(jj,ik,ichain)-xcm(jj,ik,ich)
                  dist02=dist02+tmp*tmp
                  tmp=xcm(jj,itek,ichain)-xcm(jj,itek,ich)
                  dist2=dist2+tmp*tmp
                end do
                hij0=0
                if (dist02.le.toldist2) hij0=1
                hij=0
                if (dist2.le.toldist2) hij=1
                val0=val0+hij0
                val=val+hij0*hij
              end do ! next j 
            end do ! next i
          end if  
        end do ! next ik
        if (npos.eq.initval) then
          if (val0.eq.0.and.val.eq.0) then
            write(6,*) '0/0 indetermination in CF!!'
            stop
          end if
          val0old=val0
        end if
        if (val0.ne.val0old) then
          write(6,*) 'Error in normalization for CF'
          stop
        end if
c       CF   
        ACFF=dble(val)/dble(val0)
c       elapsed time [in ps]
        ttime=dble(npos)*dble(nread)*time/1000.0d0
c       write outputfile
        write(88,298) ttime,ACFF,dlog(ACFF)
      end do ! next npos
      close(88)
C
C     VI) End of run
C
      write(6,*) 'End of run Ok!!!'
C
C     Formats
C
298   FORMAT(F8.2,1x,F9.6,1x,E14.6)
C
      end
