      include "readall.f"
C
C     Real*4 runwrap
C
      SUBROUTINE runwrap(xrf,xunrf)  
C
      implicit none
      include "params.h"
      include "dimensions.h"
      include "commons.h"
C
C     *****local variables
C
      integer iat,jat,ibond,kk
      real*4 xrf(3,maxat),xunrf(3,maxat),diff
      real*4 rboxgen(3),el2gen(3)
      common /msd1/ rboxgen,el2gen
C
      do iat = 1,nat
        do kk = 1,3
          xunrf(kk,iat) = xrf(kk,iat)
        end do
      end do
C
C     *****unwrap by bonds
C
      do ibond = 1,nbonds
        iat = bonds(1,ibond)
        jat = bonds(2,ibond)
        do kk = 1,3
          diff = xrf(kk,jat) - xrf(kk,iat)
          if (abs(diff).gt.el2gen(kk)) 
     $      diff = diff - sign(rboxgen(kk),diff)
          xunrf(kk,jat) = xunrf(kk,iat) + diff
        end do
      end do
      return
      end
C
C     ***  Main Complexes program *********
C
C     version 1.0 written by C. J. F. Solano
C
      PROGRAM complexes
      implicit none
C
      include "params.h"
      include "dimensions.h"
      include "commons.h"
C
      integer nfiles,chaintype(maxnch),numchtp,nshots
      integer iat,itype,ival,ich,jch,itmp,nch2,nptot
      integer ifiles,nat2,natoms,isnap,i,j,k,ipos,iii,ixyz
      integer nref,nlig,numref(maxnch),numlig(maxnch),nowlig(maxnch)
      integer, allocatable :: ifn(:),iln(:),numch(:)
      integer, allocatable :: numb(:)
      real*4 xx(3,maxat),xun(3,maxat),time
      real*4 rboxgen(3),el2gen(3)
      real*8 rdist,qtmp,echain(maxnch),chainmass
      real*8 xcm(3,maxnch),boxold(3)
      real*8, allocatable :: ech(:)
      character*50, allocatable :: fname(:)
      character*3 intg
      character*11 filenam
      logical ok,ok2
      common /msd1/ rboxgen,el2gen
C
C     I) Initializations
C
C     Read input.file file
      call readinit() ! fname1,nread,nskip,npchs,molec_labels,iref,ilig,
                      ! maxcoor,rmax,logsrf,recbox,rupdt,Ncell,maxfil
C     Files for analysis
      open(22,file=fname1,status="old")
      read(22,*) nfiles
      if (nfiles.lt.1) then 
        write(6,*) 'nfiles < 1'
        stop
      end if
      allocate (fname(nfiles))
      do i=1,nfiles
        read(22,'(a50)') fname(i)
        write(6,'(a50)') fname(i)
        open (30+i,file=fname(i),form='unformatted',status='old')
      end do
      write(6,*) 'Opened ',nfiles,' fort.77 (Instantaneous' 
     $         //' Coordinates) files'
      close(22)
C     Read ff.dat, connectivity.dat and coords.inp files  
      call read12() ! atom_labels,q,ncharge
      call read11() ! nch,nat,chain,natch
      call read26() ! atomtype,box,mass,ldummy
      do j = 1,3
        boxold(j)=box(j)
      end do ! next j
C     Determine possible chain (molecule) types, charges of chains, 
C     and number of chains of each type
      do ich = 1,nch
        echain(ich)=0.0d0
      end do
      do iat = 1,nat
        echain(chain(iat))=echain(chain(iat))+q(atomtype(iat))
      end do  ! next iat
      nch2=nch/Ncell 
      allocate (ech(npchs),ifn(npchs),iln(npchs),
     $          numch(npchs))
      ! chain type '1'
      ifn(1)=1
      itype=1
      itmp=natch(1)
      qtmp=echain(1)
      ich=1
      do while (ich.lt.nch2)
        ich=ich+1
        if (natch(ich).ne.itmp.or.
     $      abs(echain(ich)-qtmp).gt.0.01) then
          ! chain type 'itype'
          iln(itype)=ich-1
          ! chain type 'itype+1'
          itype=itype+1
          if (itype.gt.npchs) then
            write(6,*) 'Number of chain types is greather than ',
     $                 npchs
            stop
          end if
          ifn(itype)=ich
          itmp=natch(ich)
          qtmp=echain(ich)
        end if
      end do  ! next ich
      ! chain type 'npchs'
      if (itype.ne.npchs) then
        write(6,*) 'In automatically assignment there are ',itype,
     $             ' chain types'
        stop
      end if
      iln(npchs)=nch2
      ! Obtain 'chaintype'
      do i=1,npchs
        write(6,*) molec_labels(i),' type for chains'
        do j = 1, Ncell
          ival=(j-1)*nch2
          write(6,*) ' from ',ifn(i)+ival,' to ',iln(i)+ival
          do k = ifn(i)+ival,iln(i)+ival
            chaintype(k)=i
          end do
        end do
      end do ! next i
C
      nat2=nat/Ncell
      do i = 1,npchs
C       Determine the charge for the first chain of this type
        ech(i)=0.0d0
        numchtp=0
        do iat = 1,nat2
          if (chain(iat).eq.ifn(i)) then  ! choose the first chain of the type
            ech(i)=ech(i)+q(atomtype(iat))
            if (.not.ldummy(iat)) numchtp=numchtp+1
          end if
        end do   ! next iat
C       Check to make sure that all chain of the same type have the same charge
        do j = ifn(i),iln(i)
          if (abs(echain(j)-ech(i)).gt.1d-5)  then
            write(6,*) 'For the chain ',j,' charge = ',echain(j)
            write(6,*) 'but for the chain type ',i,' it is = ',
     $                  ech(i)
            stop
          end if 
        end do ! next j
        write(6,*) 'For the chain type ',molec_labels(i),
     $             ' the charge is ',ech(i)
        write(6,*) 'For the chain type ',molec_labels(i),
     $             ' the number of atoms is ',numchtp
      end do ! next i
C     Determine number of chains of each type and total number of chains
      ntot=0
      do i = 1,npchs
        numch(i)=iln(i)-ifn(i)+1
        numch(i)=numch(i)*Ncell
        write(6,*) 'Type ',molec_labels(i),' has ',
     $              numch(i),' molecules'
        ntot=ntot+numch(i)
      end do ! next i
      write(6,*) 'There are ',ntot,' molecules'
C     Determine number of atoms of each type
      nptot=0
      do iat = 1,nat
        itype=atomtype(iat)
        if (itype.gt.nptot) nptot=itype
      end do ! next iat
      if (nptot.ne.ncharge) then
        write(6,*) 'Number of atom types in ff.dat is',ncharge
        write(6,*) 'but now is',nptot
        stop
      end if
      write(6,*) 'There are ',ncharge,' atom types :'
      write(6,*) (atom_labels(itype),itype=1,ncharge)
      allocate (numb(ncharge))
      do itype = 1,ncharge
        numb(itype)=0
      end do ! next itype
      do iat = 1,nat
        itype=atomtype(iat)
        numb(itype)=numb(itype)+1
      end do ! next iat
C  
      write(6,*) 'atom type---number of atoms'
      do itype = 1,ncharge
        write(6,*) atom_labels(itype),numb(itype)
      end do ! next itype
      write(6,*) 'molec type---number of molecs'
      do itype = 1,npchs
        write(6,*) molec_labels(itype),numch(itype)
      end do ! next itype
C     Determine pointers and total number of atoms for complexes calculations
      nref=0
      nlig=0
      do ich = 1,nch
        itype=chaintype(ich)
        if (itype.eq.iref) then
          nref=nref+1 
          numref(nref)=ich
        end if
        if (itype.eq.ilig) then
          nlig=nlig+1
          numlig(nlig)=ich
        end if
      end do ! next ich
      nat2=0 ! total number of atoms in the complex
      ich=numref(1)
      jch=numlig(1)
      do iat = 1,nat
        if (.not.ldummy(iat).and.chain(iat).eq.ich) nat2=nat2+1
        if (.not.ldummy(iat).and.chain(iat).eq.jch) nat2=nat2+maxcoor
      end do ! next iat
C
C     II) Complexes calculation
C
      ipos=0 ! counter for positions along time
      iii=0 ! counter for two-lines block in fort.77 files
      ixyz=0 ! number of .xyz files 
      do ifiles = 31,30+nfiles
        ok=ixyz.lt.maxfil
        do while (ok)
C         Read data from fort.77 file
          ! first line
          if (recbox) then
            read(ifiles,iostat=i) natoms,time,isnap,box(1),box(2),
     $                            box(3)  
          else
            read(ifiles,iostat=i) natoms,time,isnap,box(1)
            box(2) = box(1)
            box(3) = box(1)
          end if
          if (i.eq.0) then
            if (natoms.ne.nat) then
              write(6,*) 'Wrong number of atoms in fort.77 file'
              write(6,*) natoms,' atoms is different to ',nat
              stop
            endif
            do j = 1,3
              if (boxold(j).ne.box(j)) then
                write(6,*) 'Different box sizes : ',box(j),boxold(j)
                stop
              endif
            end do 
            ! second line
            read(ifiles,iostat=i) xx 
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          endif
          if (i.eq.0) then
            ok=.true.
            iii=iii+1
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          else
            ok=.false. ! EOF
          endif
          ok=ok.and.ixyz.lt.maxfil
C         Writting .xyz files
          if (ok .and. iii.gt.nskip) then 
            ipos=ipos+1
            if (mod(ipos,nread).eq.0) then
              nshots=ipos/nread
C             Cell parameters
              do j = 1,3
                rboxgen(j)=real(box(j))
                el2gen(j)=0.5*rboxgen(j)
              end do
C             Unwrap atomic coordinates
              call runwrap(xx,xun)
C             Obtention of center of mass for each chain in a given time 
              do ich = 1,nch
                do j = 1,3
                  xcm(j,ich)=0.0d0 ! time-dependent coordinates for the center of mass
                end do
                chainmass=0.0d0
                do iat = 1,nat
                  if (.not.ldummy(iat) .and. chain(iat).eq.ich) then
                    chainmass=chainmass+mass(iat)
                    do j = 1,3
                      xcm(j,ich)=xcm(j,ich)+xun(j,iat)*mass(iat)
                    end do
                  endif
                end do
                do j = 1,3
                  xcm(j,ich)=xcm(j,ich)/chainmass
C                 Apply periodic boundary conditions
                  if (xcm(j,ich).gt.box(j)) xcm(j,ich)=xcm(j,ich)- 
     $                                                 box(j)
                  if (xcm(j,ich).lt.0.0) xcm(j,ich)=xcm(j,ich)+box(j)
                end do 
              end do ! next ich
C             Check if there are complexes for this snapshot and 
C             write .xyz file
              do i = 1,nref
                ich=numref(i)
                ival=0
                j=0
                do while (j.lt.nlig)
                  j=j+1
                  jch=numlig(j)
                  rdist=0.0d0
                  do k = 1,3
                    rdist=rdist+(xcm(k,jch)-xcm(k,ich))*
     $                          (xcm(k,jch)-xcm(k,ich))
                  end do
                  if (rdist.lt.rmax) then
                    ival=ival+1
                    nowlig(ival)=jch
                  end if
                end do
                if (ival.eq.maxcoor .and. ixyz.lt.maxfil) then
                  ixyz=ixyz+1 
                  write(intg,'(i3)') ixyz
                  filenam=intg//'file.xyz'
                  open(89,file=filenam,status='unknown')
                  write(89,*) nat2
                  write(89,'(a,1x,i3,a,1x,i3)') 
     $             '# file number:',ixyz,' snapshot:',nshots
                  do iat = 1,nat
                    ok2=.not.ldummy(iat).and.chain(iat).eq.ich ! Does 'iat' belong to reference chain?
                    j=0
                    do while (j.lt.maxcoor .and. .not.ok2)
                      j=j+1
                      jch=nowlig(j)
                      ok2=.not.ldummy(iat).and.chain(iat).eq.jch ! Does 'iat' belong to linked chain?
                    end do  
                    if (ok2) then
                      itype=atomtype(iat)
                      write(89,'(a3,5x,3F16.8)') 
     $                atom_labels(itype)(1:3),(xun(j,iat),j=1,3)
                    end if 
                  end do ! next iat
                  close(89)
                end if ! ival.eq.maxcoor
              end do ! next i  
            end if ! (mod(ipos,nread).eq.0)
          end if ! ok .and. iii.gt.nskip
        end do
        close(ifiles)
      end do ! next ifiles
C
C     IV) End of run
C
      write(6,*)
      write(6,*) 'Total number of snapshots : ',nshots
      write(6,*) 'End of run Ok!!!'
C
      end
