      include "readall.f"
C
C     Real*4 runwrap
C
      SUBROUTINE runwrap(xrf,xunrf)  ! customized
C
      implicit none
      include "params.h"
      include "dimensions.h"
      include "commons.h"
C
C     *****local variables
C
      integer iat,jat,ibond,kk
      real*4 xrf(3,maxat),xunrf(3,maxat),diff
      real*4 rboxgen,el2gen
      common /msd1/ rboxgen,el2gen
C
      do iat = 1,nat
        xunrf(1,iat) = xrf(1,iat)
        xunrf(2,iat) = xrf(2,iat)
        xunrf(3,iat) = xrf(3,iat)
      end do
C
C     *****unwrap by bonds
C
      do ibond = 1,nbonds
        iat = bonds(1,ibond)
        jat = bonds(2,ibond)
        do kk = 1,3
          diff = xrf(kk,jat) - xrf(kk,iat)
          if (abs(diff) .gt. el2gen)
     $      diff = diff - sign(rboxgen,diff)
          xunrf(kk,jat) = xunrf(kk,iat) + diff
        end do
      end do
      return
      end
C
C     LINEAR LEAST SQUARE REGRESSION
C
      SUBROUTINE linear_regression(initval,npoints,ntypes,x,y,ixx)

      implicit none
      integer, intent(in) :: initval,npoints,ntypes,ixx
      real*8, intent(in), dimension(npoints) :: x
      real*8, intent(in), dimension(ntypes,npoints) :: y
C     Note that npoints > (initval+1) and ntypes > 0. Thus, these values 
C     have to be tested before calling this routine.
      
C
C     *****local variables
C
      integer :: i,j
      real*8 :: points,a,b,r2,sea,seb
      real*8 :: xav,yav,ssxx,ssyy,ssxy,s

      points = dble(npoints-initval+1)
C
C     I) OBTENTION OF AVERAGE VALUES
C
      xav = 0.0d0
      do j = initval, npoints
        xav = xav + x(j)
      enddo
      xav = xav/points  
C
C     II) OBTENTION OF SUMS OF SQUARES
C
      ssxx = 0.0d0
      do j = initval, npoints
        ssxx = ssxx + x(j)*x(j)
      enddo
      ssxx = ssxx - points*xav*xav

      do i = 1, ntypes
C
C       I) OBTENTION OF AVERAGE VALUES (cont.)
C
        yav = 0.0d0
        do j = initval, npoints
          yav = yav + y(i,j)
        enddo
        yav = yav/points
C
C       II) OBTENTION OF SUMS OF SQUARES (cont.)
C
        ssyy = 0.0d0
        ssxy = 0.0d0
        do j = initval, npoints
          ssyy = ssyy + y(i,j)*y(i,j) 
          ssxy = ssxy + x(j)*y(i,j)
        enddo
        ssyy = ssyy - points*yav*yav   
        ssxy = ssxy - points*xav*yav
C
C       III) OBTENTION OF REGRESSION COEFFICIENT b AND a
C
        b = ssxy/ssxx
        a = yav - b*xav
C
C       IV) OBTENTION OF CORRELATION COEFFICIENT r2 AND 
C           STANDARD ERRORS (sea,seb)
C
        r2 = b*ssxy/ssyy
        s = sqrt((ssyy-b*ssxy)/dble(npoints-initval-1))
        sea = s*sqrt(1.0d0/points+xav*xav/ssxx)
        seb = s/sqrt(ssxx)
C
C       V) WRITE INFORMATION         
C
        write(91,*) i,' molecule type'
        if (ixx.eq.1) then ! total
          write(91,*) 'Self-diffusion constant = ',b/6.0d0,
     $                ' Angs**2/ps'
        else if (ixx.eq.2) then ! cartesian components
          write(91,*) 'Self-diffusion constant = ',b/2.0d0,
     $                ' Angs**2/ps'
        endif
        write(91,*) 'Y-intercept = ',a,' Angs**2'
        write(91,*) 'Correlation coefficient = ',r2
        write(91,*) 'Standard errors (y-intercept,slope) = ',sea,seb

      enddo

C
C     END OF SUBROUTINE
C
      return
      end       
C
C     ***  Main MSD program *********
C
C     version v1.5 written by C. J. F. Solano
C
      PROGRAM msd
      implicit none
C
      include "params.h"
      include "dimensions.h"
      include "commons.h"
C
      integer maxshots
      parameter (maxshots=30101) ! maximum number of snaps
      real*8 xcm(3,maxshots,maxnch)
      real*4 xreal(3,maxat),xrealold(3,maxat)
      real*4 xx(3,maxat),xold(3,maxat),xun(3,maxat),aa
      real*4 rboxgen,el2gen
      real*4 time
      real*8, allocatable :: dmsd(:),xav(:,:)   
      real*8, allocatable :: dmsdx(:),dmsdy(:),dmsdz(:)
      real*8, allocatable :: disp(:),ech(:)
      integer, allocatable :: ifn(:),iln(:),numberp(:) 
      real*8 rmsd(maxnch)
      real*8 dka(3),echain(maxnch)
      real*8 dkb(3),qtmp,dispmax,tmp,chainmass,rtmp,ttime
      real*8 aterm,bterm,const
      real*8, allocatable :: elaptime(:),msdtot(:,:),msdx(:,:),
     $                       msdy(:,:),msdz(:,:)
      integer ntot,norigin,numchtp
      integer nfiles
      integer ichain,iat,itype,itmp,i,jj,ipt
      integer ifiles,natoms,isnap,j,ipos,ich,nshots,
     $  npos,ik,itek,i1,j1,iii
      character*50, allocatable :: fname(:)
      logical ok
      common /msd1/rboxgen,el2gen
C
C     I) Initializations
C
C     Files for analysis
      call readinit()
      open(22,file=fname1,status="old")
      read(22,*) nfiles
      if (nfiles.lt.1) then
        write(6,*) 'nfiles < 1'
        stop
      endif
      allocate (fname(nfiles))
      do i=1,nfiles
        read(22,'(a50)') fname(i)
        write(6,'(a50)') fname(i)
        open (30+i,file=fname(i),form='unformatted',status='old')
      end do
      write(6,*) 'Opened ',nfiles,' fort.77 (Instantaneous' 
     $         //' Coordinates) files'
      close(22)
C        
      call read12()
      call read11()
      call read26()
C
C    II) Determine possible chain (molecule) types, charges of chains, 
C        and number of chains of each type
C
      do ichain=1,maxnch
        echain(ichain)=0.0d0
      end do
      do iat=1,nat
        echain(chain(iat))=echain(chain(iat))+q(atomtype(iat))
      end do  ! next iat
C
      allocate (dmsd(nptypes),xav(3,nptypes),dmsdx(nptypes),
     $          dmsdy(nptypes),dmsdz(nptypes))
      allocate (disp(nptypes),ech(nptypes),ifn(nptypes),iln(nptypes),
     $          numberp(nptypes))
      ! chain type '1'
      ifn(1)=1
      itype=1
      itmp=natch(1)
      qtmp=echain(1)
      ichain=1
      do while (ichain.lt.maxnch)
        ichain=ichain+1
        if (natch(ichain).ne.itmp.or.
     $      abs(echain(ichain)-qtmp).gt.0.01) then
          ! chain type 'itype'
          iln(itype)=ichain-1
          ! chain type 'itype+1'
          itype=itype+1
          if (itype.gt.nptypes) then
            write(6,*) 'Number of chain types is greather than ',
     $                 nptypes
            stop
          endif
          ifn(itype)=ichain
          itmp=natch(ichain)
          qtmp=echain(ichain)
        endif
      end do  ! next ichain
      ! chain type 'itype'
      iln(itype)=maxnch
      if (itype.ne.nptypes) then
        write(6,*) 'In automatically assignment there are ',itype,
     $             ' chain types'
        stop  
      endif
      do i=1,nptypes
        write(6,*) labeltyp(i),' type for chains from ',ifn(i),
     $             ' to ',iln(i)     
      end do
C
      do i=1,nptypes
C       Determine the charge for the first chain of this type
        ech(i)=0.0d0
        numchtp=0
        do iat=1,nat
          if (chain(iat).eq.ifn(i)) then  ! choose the first chain of the type
            ech(i)=ech(i)+q(atomtype(iat))
            if (.not.ldummy(iat)) numchtp=numchtp+1
          endif
        end do   ! next iat
C       Check to make sure that all chain of the same type have the same charge
        do jj=ifn(i),iln(i)
          if (abs(echain(jj)-ech(i)).gt.1d-5)  then
            write(6,*) 'For the chain ',jj,' charge = ',echain(jj)
            write(6,*) 'but for the chain type ',i,' it is = ',
     $                  ech(i)
            stop
          endif 
        end do ! next jj
        write(6,*) 'For the chain type ',labeltyp(i),
     $             ' the charge is ',ech(i)
        write(6,*) 'For the chain type ',labeltyp(i),
     $             ' the number of atoms is ',numchtp
      end do ! next i
C     Determine number of chains of each type and total number of chains
      ntot=0
      do ipt=1,nptypes
        numberp(ipt)=iln(ipt)-ifn(ipt)+1
        ntot=ntot+numberp(ipt)
        write(6,*) labeltyp(ipt),' type has ',numberp(ipt),' molecules'
      end do
      write(6,*) 'There are ',ntot,' molecules'
C
C     III) Center of mass calculations
C
      dispmax=0.0d0 ! maximum displacement
      ipos=0 ! counter for positions along time
      iii=0 ! counter for two-lines block in fort.77 files
      do ifiles = 31,30+nfiles
        ok = .true.
        do while (ok)
C         Read data from fort.77 file
          read(ifiles,iostat=i) natoms,time,isnap,box ! first line 
          if (i.eq.0) then
            if (natoms.ne.nat) then
              write(6,*) 'Wrong number of atoms in fort.77 file'
              write(6,*) natoms,' atoms is different to ',nat
              stop
            endif
            if (iii.eq.0) boxold=box
            if (box.ne.boxold) then
              write(6,*) 'Different box sizes : ',box,boxold
              stop
            end if
            read(ifiles,iostat=i) xx ! second line
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          endif
          if (i.eq.0) then
            ok = .true.
            iii=iii+1
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          else
            ok = .false. ! EOF
          endif
          if (ok .and. iii.gt.nskip) then
C           Periodic cubic cell
            rboxgen=real(box)
            el2gen=0.5*rboxgen
C           Treatment of the Cartesian coordinates
            ipos=ipos+1
            if (ipos.eq.1) then
              do iat=1,nat
                do j=1,3
                  xreal(j,iat)=xx(j,iat)
                end do
              end do
            else
              do iat=1,nat
                tmp=0.0d0
                do j=1,3
                  aa=xx(j,iat)-xold(j,iat)
                  if (abs(aa).gt.el2gen) 
     $              aa=aa-sign(rboxgen,aa)
                  tmp=tmp+aa*aa
                  xreal(j,iat)=xrealold(j,iat)+aa
                end do  ! next j  
                if (tmp.gt.dispmax) dispmax=tmp
              end do    ! next iat
            endif       ! (ipos.eq.1)
            do iat=1,nat
              do j=1,3
                xold(j,iat)=xx(j,iat)
                xrealold(j,iat)=xreal(j,iat)
              end do
            end do 
            call runwrap(xreal,xun)
C           Obtention of center of mass for each chain in each time
            if (mod(ipos,nread).eq.0) then
              nshots=ipos/nread
              if (maxshots.lt.nshots) then
                write(6,*) 'Increase the maxshots'
                write(6,*) 'Current maxshots = ',maxshots
                stop
              endif
              do ich=1,maxnch
                do j=1,3
                  xcm(j,nshots,ich)=0.0d0 ! time-dependent coordinates for the center of mass
                end do
                chainmass=0.0d0
                do iat=1,nat
                  if (chain(iat).eq.ich) then
                    chainmass=chainmass+mass(iat)
                    do j=1,3
                      xcm(j,nshots,ich)=xcm(j,nshots,ich)+
     $                                 xun(j,iat)*mass(iat)
                    end do
                  endif
                end do
                do j=1,3
                  xcm(j,nshots,ich)=xcm(j,nshots,ich)/chainmass
                end do
              end do ! next ich
            endif ! (mod(ipos,nread).eq.0)
          endif ! (ok .and. iii.ge.nskip)
        end do
        write(6,*) 'fort.77 file ',ifiles,':'
        write(6,*) natoms,' atoms, ',time,' time step (fs), ',box,
     $             ' cell size (Angs.)' 
        write(6,*) isnap,' final snapshot file, ',ipos,
     $             ' actual snapshot'
        close(ifiles)
      end do ! next ifiles
      if (nshots.lt.2) then
        write(6,*) 'nshots < 2'
        stop
      endif
      write(6,*) 'Maximum displacement (Angs.)',dsqrt(dispmax)
C
C     IV) Generation of 'check.msd' file data
C
      open (2,file='check.msd',status='unknown')
      write(2,*) '# ipos, av_displ., x_av(ipt), y_av(ipt), z_av(ipt)'
      do npos=2,nshots
        do ipt=1,nptypes
          dmsd(ipt)=0.0d0
          do j=1,3
            xav(j,ipt)=0.0d0
          end do 
          do i=ifn(ipt),iln(ipt)
            tmp=0.0d0
            do j=1,3
              tmp=tmp+(xcm(j,npos,i)-xcm(j,npos-1,i))**2
              xav(j,ipt)=xav(j,ipt)+xcm(j,npos,i)-xcm(j,1,i)
            end do
            dmsd(ipt)=dmsd(ipt)+tmp
          end do ! next i 
          do j=1,3
            xav(j,ipt)=xav(j,ipt)/dble(numberp(ipt))
          end do
        end do ! next ipt
        write(2,'(I9,30F9.3)') npos,
     $   (dmsd(ipt)/dble(numberp(ipt)),
     $   ipt=1,nptypes),(xav(1,ipt),ipt=1,nptypes),
     $   (xav(2,ipt),ipt=1,nptypes),(xav(3,ipt),ipt=1,nptypes)
      end do      ! next npos
      close(2)
C
C     V) Mean-square displacement and collective charge transport       
C
C     Obtention of time origins
      norigin=nshots/2
      if (initval.gt.norigin) then
        write(6,*) 'Initial elapsed time value is greather than'
     $           //' maximum value of ',norigin
        stop
      endif
      if (endval.gt.norigin) then
        write(6,*) 'Final elapsed time value is greather than'
     $           //' maximum value of ',norigin
        stop
      endif
      write (6,*) 'There are ',nshots,' times'
      write (6,*) 'There are ',norigin,' time origins'
      if (Qregr .and. endval.le.(initval+1)) then
        write(6,*) 'Linear least square regression of MD is turned'
     $           //' off because of number of origins <= 2'
        Qregr=.false.
      endif
      if (Qregr) then
        allocate (elaptime(endval),msdtot(nptypes,endval),
     $            msdx(nptypes,endval),msdy(nptypes,endval),
     $            msdz(nptypes,endval))
      endif
C     Open outputfiles
      if (Qout) then
        open (88,file=fname2,status='new')
        open (89,file=fname3,status='new')
        open (90,file=fname4,status='new')
        write(88,'(a,1x,f8.5)') "# Maximum displacement (Angs.)",
     $                          dsqrt(dispmax)
        do i=1,nfiles
          write(88,'("# fort.77 file = ",a50)') fname(i)
          write(89,'("# fort.77 file = ",a50)') fname(i)
          write(90,'("# fort.77 file = ",a50)') fname(i)
        end do
        write(88,'(a,i10,a,10i6)') "# v2.1:nfiles ",nfiles," chains ",
     $        (ifn(i),iln(i),i=1,nptypes)
        write(89,'(a,i10,a,10i6)') "# v2.1:nfiles ",nfiles," chains ",
     $        (ifn(i),iln(i),i=1,nptypes)
        write(90,'(a,i10,a,10i6)') "# v2.1:nfiles ",nfiles," chains ",
     $        (ifn(i),iln(i),i=1,nptypes)
        write(88,'(a)') "# time -> elapsed time [ps]"
        write(88,'(a)') "# MSD_(x,y,z) -> (x,y,z) component of" 
     $            //" mean-square displacement for each chain"
     $            //" type [Angs**2]"
        write(88,'(a)') "# MSD_tot -> mean-square displacement for each"
     $            //" chain type [Angs**2]"
        write(88,'(a)') "# sigma -> standard deviation for each chain" 
     $            //" type [Angs**2]"
        write(88,'(a)') "# time---MSD_x---MSD_y---MSD_z---MSD_tot---"
     $                //"sigma"
        write(89,'(a)') "# time -> elapsed time [ps]"
        write(89,'(a)') "# D_(x,y,z) -> (x,y,z) component of" 
     &                //" self-diffusion constant [Angs**2/ps]"
        write(89,'(a)') "# D -> self-diffusion constant [Angs**2/ps]"
        write(89,'(a)') "# sigma -> standard deviation for each chain" 
     $            //" type [Angs**2/ns]"
        write(89,'(a)') "# time---D_x---D_y---D_z---D---sigma"
        write(90,'(a,f8.2)') "# Temperature [K] = ", temper
        write(90,'(a)') "# time -> elapsed time [ps]"
        write(90,'(a)') "# lambtot -> Collective total charge transport"
     $            //" [S/cm]"
        write(90,'(a)') "# lambself -> Charge transport due to"
     $            //"  self-diffusion [S/cm]"
        write(90,'(a)') "# alpha -> degree of uncorrelated motion"
        write(90,'(a)') "# time---lambtot---lambself---alpha"
      end if
C
      do npos=initval,endval ! npos is the elapsed time 
        aterm = 0.0d0
        bterm = 0.0d0
        do ipt=1,nptypes
          dmsd(ipt)=0.0d0 
          dmsdx(ipt)=0.0d0
          dmsdy(ipt)=0.0d0
          dmsdz(ipt)=0.0d0
        end do
        do ich=1,maxnch
          rmsd(ich)=0.0d0
        end do
        do ik=1,norigin !ik is the starting position
          itek=ik+npos ! itek is the final position
C         Collective charge transport
          ! Cross terms
          i1=0
          do while (i1.lt.(maxnch-1))
            i1=i1+1
            j1=i1
            do while (j1.lt.maxnch)
              j1=j1+1
              do j=1,3
                dka(j)=xcm(j,itek,i1)-xcm(j,ik,i1)
                dkb(j)=xcm(j,itek,j1)-xcm(j,ik,j1)
              end do
              aterm=aterm+2.0d0*echain(i1)*echain(j1)*(dka(1)*dkb(1)+
     $             dka(2)*dkb(2)+dka(3)*dkb(3))
            end do
          end do
          ! Diagomal terms
          do i1=1,maxnch
            do j=1,3
              dka(j)=xcm(j,itek,i1)-xcm(j,ik,i1)
            end do
            aterm=aterm+echain(i1)*echain(i1)*(dka(1)*dka(1)+
     $            dka(2)*dka(2)+dka(3)*dka(3))
            bterm=bterm+echain(i1)*echain(i1)*(dka(1)*dka(1)+
     $            dka(2)*dka(2)+dka(3)*dka(3))
          end do
C         Mean-square displacement
          do ipt=1,nptypes
            do i=ifn(ipt),iln(ipt)
              do j=1,3
                dka(j)=xcm(j,itek,i)-xcm(j,ik,i)
              end do
              rtmp=dka(1)*dka(1)+dka(2)*dka(2)+dka(3)*dka(3)
              rmsd(i)=rmsd(i)+rtmp
              dmsd(ipt)=dmsd(ipt)+rtmp
              dmsdx(ipt)=dmsdx(ipt)+dka(1)*dka(1)
              dmsdy(ipt)=dmsdy(ipt)+dka(2)*dka(2)
              dmsdz(ipt)=dmsdz(ipt)+dka(3)*dka(3)
            end do ! next i
          end do ! next ipt
        end do ! next ik
        do ipt=1,nptypes          
          dmsd(ipt)=dmsd(ipt)/(dble(numberp(ipt))
     $                              *dble(norigin))
          dmsdx(ipt)=dmsdx(ipt)/(dble(numberp(ipt))
     $                              *dble(norigin))
          dmsdy(ipt)=dmsdy(ipt)/(dble(numberp(ipt))
     $                              *dble(norigin))
          dmsdz(ipt)=dmsdz(ipt)/(dble(numberp(ipt))
     $                              *dble(norigin))
          rtmp=dmsdx(ipt)+dmsdy(ipt)+dmsdz(ipt)
          if (abs(dmsd(ipt)-rtmp).gt.1.0d-10) then
            write(6,*) '**** warning ****'
            write(6,*) 'Error in MSD calculation'
            write(6,*) 'MSD from direct expression = ',dmsd(ipt)
            write(6,*) 'MSD from different cartesian components = ',
     $                  rtmp
          endif          
          disp(ipt)=0.0d0
          if (Qregr) then
            msdtot(ipt,npos)=dmsd(ipt)
            msdx(ipt,npos)=dmsdx(ipt)
            msdy(ipt,npos)=dmsdy(ipt)
            msdz(ipt,npos)=dmsdz(ipt)
          endif
        end do ! next ipt
        do ich=1,maxnch
          rmsd(ich)=rmsd(ich)/dble(norigin)
        end do
        ! Standard deviation
        do ipt=1,nptypes
          do ich=ifn(ipt),iln(ipt)
            disp(ipt)=disp(ipt)+(rmsd(ich)-dmsd(ipt))**2
          end do
        end do
        do ipt=1,nptypes
          disp(ipt)=dsqrt(disp(ipt)/(dble(numberp(ipt))))
        end do
c       elapsed time [in ps]
        ttime= dble(npos)*dble(nread)*time/1000.0d0
        if (Qregr) elaptime(npos)=ttime
c
        const=(1.602176565d0*1.602176565d0*1.0d5)/(6.0d0*ttime
     $         *box*box*box*1.3806488d0*temper) 
        const=const/(dble(ntot)*dble(norigin))
c       write outputfiles
        if (Qout) then
          write(88,298) ttime,
     $      (dmsdx(j),j=1,nptypes),
     $      (dmsdy(j),j=1,nptypes),
     $      (dmsdz(j),j=1,nptypes),
     $      (dmsd(j),j=1,nptypes),
     $      (disp(j),j=1,nptypes)       
          write(89,298) ttime,
     $      (dmsdx(j)/(2.0d0*ttime),j=1,nptypes),
     $      (dmsdy(j)/(2.0d0*ttime),j=1,nptypes),
     $      (dmsdz(j)/(2.0d0*ttime),j=1,nptypes),
     $      (dmsd(j)/(6.0d0*ttime),j=1,nptypes),
     $      (disp(j)/(6.0d0*ttime),j=1,nptypes)
          write(90,299) ttime,aterm*const,bterm*const,aterm/bterm
        end if
      end do ! next npos
      close(88)
      close(89)
      close(90)
C
C     VI) Linear least square regression of MSD
C
      if (Qregr) then
        open (91,file=fname5,status='new')
        write(91,*) 'Linear least square regression of MSD'
        write(91,*) '======================================'
        write(91,*)
        write(91,*) 'Total self-diffusion constant analysis'
        write(91,*) '--------------------------------------'
        call linear_regression(initval,endval,nptypes,elaptime,
     $                         msdtot,1)
        write(91,*)
        write(91,*) 'Component self-diffusion constant analysis'
        write(91,*) '------------------------------------------'
        write(91,*)
        write(91,*) 'X-component:'
        call linear_regression(initval,endval,nptypes,elaptime,
     $                         msdx,2)
        write(91,*) 'Y-component:'
        call linear_regression(initval,endval,nptypes,elaptime,
     $                         msdy,2) 
        write(91,*) 'Z-component:' 
        call linear_regression(initval,endval,nptypes,elaptime,
     $                         msdz,2) 
        close(91)  
      endif
C
C     VII) End of run
C
      write(6,*) 'End of run Ok!!!'
C
C     Formats
C
298   FORMAT(F8.2,1x,120E12.4)
299   FORMAT(F8.2,1x,3E12.4)
C
      end
