      include "readall.f"
C
C     Real*4 runwrap
C
      SUBROUTINE runwrap(xrf,xunrf)  ! customized
C
      implicit none
      include "params.h"
      include "dimensions.h"
      include "commons.h"
C
C     *****local variables
C
      integer iat,jat,ibond,kk
      real*4 xrf(3,maxat),xunrf(3,maxat),diff
      real*4 rboxgen,el2gen
      common /msd1/ rboxgen,el2gen
C
      do iat = 1,nat
        xunrf(1,iat) = xrf(1,iat)
        xunrf(2,iat) = xrf(2,iat)
        xunrf(3,iat) = xrf(3,iat)
      end do
C
C     *****unwrap by bonds
C
      do ibond = 1,nbonds
        iat = bonds(1,ibond)
        jat = bonds(2,ibond)
        do kk = 1,3
          diff = xrf(kk,jat) - xrf(kk,iat)
          if (abs(diff) .gt. el2gen)
     $      diff = diff - sign(rboxgen,diff)
          xunrf(kk,jat) = xunrf(kk,iat) + diff
        end do
      end do
      return
      end
C
C     ***  Main Radial Distribution Function program *********
C
C     version 2.8 written by C. J. F. Solano
C
      PROGRAM rdf
      implicit none
C
      include "params.h"
      include "dimensions.h"
      include "commons.h"
C
      integer nptot,nptypes,nrmax,new_type(maxcharges)
      real*8 rdist,qtmp,echain(maxnch),chainmass
      real*8 delta3,r,r2,xcm(3,maxnch)
      real*4 xx(3,maxat),xun(3,maxat)
      real*4 time
      real*4 rboxgen,el2gen,rrmax
      real*8 xtemp,ytemp,ztemp,rmax2
      real*8, allocatable :: dist(:),volume(:),gr(:),rjat(:)
      real*8, allocatable :: grtot(:),rjch(:)
      real*8, allocatable :: ech(:),totnumb(:,:,:),totnumbch(:,:,:)
      integer nfiles,chaintype(maxnch),numchtp
      integer iat,jat,itype,jtype,nr,id,idch,ival,ich,ntot,itmp
      integer ifiles,natoms,isnap,i,j,ipos,nshots,iii
      integer ix,iy,iz,limit
      integer, allocatable :: ifn(:),iln(:),numch(:)
      integer, allocatable :: numb(:),typ(:,:),shell(:),pointer(:,:,:)
      integer, allocatable :: typch(:,:),shch(:),pointch(:,:,:)
      character*50, allocatable :: fname(:)
      character*5 atm_name(maxcharges)
      logical ok,ok2
      common /msd1/ rboxgen,el2gen
C
C     I) Initializations
C
      pi = dacos(-1.0d0)
C     Read input.file file
      call readinit()
C     Files for analysis
      open(22,file=fname1,status="old")
      read(22,*) nfiles
      if (nfiles.lt.1) then 
        write(6,*) 'nfiles < 1'
        stop
      endif
      allocate (fname(nfiles))
      do i=1,nfiles
        read(22,'(a50)') fname(i)
        write(6,'(a50)') fname(i)
        open (30+i,file=fname(i),form='unformatted',status='old')
      end do
      write(6,*) 'Opened ',nfiles,' fort.77 (Instantaneous' 
     $         //' Coordinates) files'
      close(22)
C     Read ff.dat, connectivity.dat and coords.inp files  
      call read12() ! atom_labels,q,typedummy
      call read11() ! nat,chain,natch
      call read26() ! atomtype,box,mass,ldummy
C     Determine possible chain (molecule) types, charges of chains, 
C     and number of chains of each type
      do ich=1,maxnch
        echain(ich)=0.0d0
      end do
      do iat=1,nat
        echain(chain(iat))=echain(chain(iat))+q(atomtype(iat))
      end do  ! next iat
      allocate (ech(npchs),ifn(npchs),iln(npchs),
     $          numch(npchs))
      ! chain type '1'
      ifn(1)=1
      itype=1
      itmp=natch(1)
      qtmp=echain(1)
      ich=1
      do while (ich.lt.maxnch)
        ich=ich+1
        if (natch(ich).ne.itmp.or.
     $      abs(echain(ich)-qtmp).gt.0.01) then
          ! chain type 'itype'
          iln(itype)=ich-1
          ! chain type 'itype+1'
          itype=itype+1
          if (itype.gt.npchs) then
            write(6,*) 'Number of chain types is greather than ',
     $                 npchs
            stop
          endif
          ifn(itype)=ich
          itmp=natch(ich)
          qtmp=echain(ich)
        endif
      end do  ! next ich
      ! chain type 'itype'
      iln(itype)=maxnch
      if (itype.ne.npchs) then
        write(6,*) 'In automatically assignment there are ',itype,
     $             ' chain types'
        stop  
      endif
      do i=1,npchs
        write(6,*) molec_labels(i),' type for chains from ',ifn(i),
     $             ' to ',iln(i)     
      end do
C
      do i=1,npchs
C       Determine the charge for the first chain of this type
        ech(i)=0.0d0
        numchtp=0
        do iat=1,nat
          if (chain(iat).eq.ifn(i)) then  ! choose the first chain of the type
            ech(i)=ech(i)+q(atomtype(iat))
            if (.not.ldummy(iat)) numchtp=numchtp+1
          endif
        end do   ! next iat
C       Check to make sure that all chain of the same type have the same charge
C       and obtain 'chaintype'
        do j=ifn(i),iln(i)
          if (abs(echain(j)-ech(i)).gt.1d-5)  then
            write(6,*) 'For the chain ',j,' charge = ',echain(j)
            write(6,*) 'but for the chain type ',i,' it is = ',
     $                  ech(i)
            stop
          endif 
          chaintype(j)=i
        end do ! next j
        write(6,*) 'For the chain type ',molec_labels(i),
     $             ' the charge is ',ech(i)
        write(6,*) 'For the chain type ',molec_labels(i),
     $             ' the number of atoms is ',numchtp
      end do ! next i
C     Determine number of chains of each type and total number of chains
      ntot=0
      do i=1,npchs
        numch(i)=iln(i)-ifn(i)+1
        write(6,*) 'Type ',molec_labels(i),' has ',
     $              numch(i),' molecules'
        ntot=ntot+numch(i)
      end do
      write(6,*) 'There are ',ntot,' molecules'
C     Initializations for RDFs and CM RDFs calculations
      rrmax=real(rmax)
      rmax2=rmax*rmax
      delta3=delta*delta*delta
      nrmax=int(rmax/delta)
      write(6,*) 'Maximum number of shells = ',nrmax
      allocate (dist(nrmax),volume(nrmax))
      nptot=0
      do iat=1,nat
        itype=atomtype(iat)
        if (itype.gt.nptot) nptot=itype
      enddo
      if (nptot.ne.maxcharges) then
        write(6,*) 'Number of atom types in params.h is',maxcharges
        write(6,*) 'but now is',nptot
        stop
      endif
      nptypes=0
      do itype=1,maxcharges
        if (typedummy(itype).eq.0) then
          nptypes=nptypes+1
          new_type(itype)=nptypes
          atm_name(nptypes)=atom_labels(itype)
        endif
      enddo
      write(6,*) 'There are ',nptypes,' atom types :'
      write(6,*) (atm_name(itype),itype=1,nptypes)
      allocate (numb(nptypes))
      allocate (pointer(nptypes,nptypes,nrmax))
      allocate (pointch(npchs,npchs,nrmax))
      id=nrmax*nptypes*nptypes
      idch=nrmax*npchs*npchs
      allocate (shell(id),typ(id,2))
      allocate (rjat(id),gr(id))
      allocate (shch(idch),typch(idch,2))
      allocate (rjch(idch),grtot(idch))
      write(6,*) 'shell---distance [Angs.]---volume [Angs.**3]'
      ival=0
      do nr=1,nrmax
        dist(nr)=(nr-0.5d0)*delta
        r=dble(nr)
        r2=r*r
        volume(nr)=4.0d0*pi*delta3*(r2-r+1.0d0/3.0d0)
        write(6,*) nr,dist(nr),volume(nr)
        do itype=1,nptypes
          if (nr.eq.nrmax) numb(itype)=0
          do jtype=1,nptypes
            ival=ival+1
            typ(ival,1)=itype
            typ(ival,2)=jtype 
            shell(ival)=nr 
            pointer(itype,jtype,nr)=ival 
            rjat(ival)=0.0d0
          enddo ! next jtype
        enddo ! next itype
      enddo ! next nr
      do iat=1,nat
        if (.not.ldummy(iat)) then 
          i=atomtype(iat)
          itype=new_type(i)
          numb(itype)=numb(itype)+1
        end if
      enddo
      ival=0
      do nr=1,nrmax
        do itype=1,npchs
          do jtype=1,npchs
            ival=ival+1
            typch(ival,1)=itype
            typch(ival,2)=jtype 
            shch(ival)=nr 
            pointch(itype,jtype,nr)=ival 
            rjch(ival)=0.0d0
          enddo ! next jtype
        enddo ! next itype
      enddo ! next nr
      write(6,*) 'atom type---number of atoms---'
     $         //'density [Angs.**(-3)]'
      do itype=1,nptypes
        write(6,*) atm_name(itype),numb(itype),
     $             numb(itype)/(box*box*box)
      enddo
      write(6,*) 'molec type---number of molecs---'
     $         //'density [Angs.**(-3)]'
      do itype=1,npchs
        write(6,*) molec_labels(itype),numch(itype),
     $             numch(itype)/(box*box*box)
      enddo
C
C     II) Radial distribution calculation
C
      ipos=0 ! counter for positions along time
      iii=0 ! counter for two-lines block in fort.77 files
      do ifiles = 31,30+nfiles
        ok = .true.
        do while (ok)
C         Read data from fort.77 file
          read(ifiles,iostat=i) natoms,time,isnap,box ! first line 
          if (i.eq.0) then
            if (natoms.ne.nat) then
              write(6,*) 'Wrong number of atoms in fort.77 file'
              write(6,*) natoms,' atoms is different to ',nat
              stop
            endif
            if (iii.eq.0) boxold=box
            if (boxold.ne.box) then
              write(6,*) 'Different box sizes : ',box,boxold
              stop
            endif
            read(ifiles,iostat=i) xx ! second line
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          endif
          if (i.eq.0) then
            ok = .true.
            iii=iii+1
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          else
            ok = .false. ! EOF
          endif
          if (ok .and. iii.gt.nskip) then 
            ipos=ipos+1
C           Obtention radial distribution function for each snapshot
            if (mod(ipos,nread).eq.0) then
              nshots=ipos/nread
              limit = int(rmax/box) + 1
C             Periodic cubic cell
              rboxgen=real(box)
              el2gen=0.5*rboxgen
              call runwrap(xx,xun)
C             Obtention of center of mass for each chain in a given time 
              do ich=1,maxnch
                do j=1,3
                  xcm(j,ich)=0.0d0 ! time-dependent coordinates for the center of mass
                end do
                chainmass=0.0d0
                do iat=1,nat
                  if (chain(iat).eq.ich) then
                    chainmass=chainmass+mass(iat)
                    do j=1,3
                      xcm(j,ich)=xcm(j,ich)+xun(j,iat)*mass(iat)
                    end do
                  endif
                end do
                do j=1,3
                  xcm(j,ich)=xcm(j,ich)/chainmass
C                 Apply periodic boundary conditions
                  if (xcm(j,ich).gt.box) xcm(j,ich)=xcm(j,ich)-box
                  if (xcm(j,ich).lt.0.0) xcm(j,ich)=xcm(j,ich)+box
                end do
              end do ! next ich
C             RDFs
              do iat=1,nat
                if (.not.ldummy(iat)) then
                  i=atomtype(iat)
                  itype=new_type(i)
                  do jat=1,nat
                    if (.not.ldummy(jat)) then
                      do ix=-limit,limit
                        do iy=-limit,limit
                          do iz=-limit,limit
                            ok2=jat.eq.iat.and.ix.eq.0.and.iy.eq.0
     $                          .and.iz.eq.0
                            if (.not.ok2) then 
                              xtemp = xx(1,jat)+ix*rboxgen-xx(1,iat)
                              ytemp = xx(2,jat)+iy*rboxgen-xx(2,iat)
                              ztemp = xx(3,jat)+iz*rboxgen-xx(3,iat)
                              rdist = xtemp*xtemp + ytemp*ytemp +
     $                                ztemp*ztemp
                              if (rdist.lt.rmax2) then
                                j=atomtype(jat)
                                jtype=new_type(j)
                                rdist = dsqrt(rdist)
                                nr = int(rdist/delta) + 1
                                ival=pointer(itype,jtype,nr)
C                               number of 'jtype' atoms at distance 
C                               between [(nr-1)*delta,nr*delta] 
C                               from a given 'itype' atom 
                                rjat(ival)=rjat(ival)+1.0d0
                              endif
                            endif 
                          enddo ! next iz
                        enddo ! next iy
                      enddo ! next ix                
                    endif
                  enddo ! next jat
                endif
              enddo ! nex iat
C             CM RDFs
              do iat=1,maxnch
                itype=chaintype(iat)
                do jat=1,maxnch
                  do ix=-limit,limit
                    do iy=-limit,limit
                      do iz=-limit,limit
                        ok2=jat.eq.iat.and.ix.eq.0.and.iy.eq.0
     $                      .and.iz.eq.0
                        if (.not.ok2) then
                          xtemp = xcm(1,jat)+ix*box-xcm(1,iat)
                          ytemp = xcm(2,jat)+iy*box-xcm(2,iat)
                          ztemp = xcm(3,jat)+iz*box-xcm(3,iat)
                          rdist = xtemp*xtemp + ytemp*ytemp +
     $                            ztemp*ztemp
                          if (rdist.lt.rmax2) then
                            jtype=chaintype(jat)
                            rdist = dsqrt(rdist)
                            nr = int(rdist/delta) + 1 
                            ival=pointch(itype,jtype,nr)
C                           number of 'jtype' molecules at distance 
C                           between [(nr-1)*delta,nr*delta] 
C                           from a given 'itype' molecule
                            rjch(ival)=rjch(ival)+1.0d0
                          endif
                        endif
                      enddo ! next iz
                    enddo ! next iy
                  enddo ! next ix
                enddo ! next jat
              enddo ! nex iat
            endif ! (mod(ipos,nread).eq.0)
          endif ! ok .and. iii.gt.nskip
        enddo
        write(6,*) 'fort.77 file ',ifiles,':'
        write(6,*) natoms,' atoms, ',time,' time step (fs), ',box,
     $             ' cell size (Angs.)' 
        write(6,*) isnap,' final snapshot file, ',ipos,
     $             ' actual snapshot'
        close(ifiles)
      end do ! next ifiles
C      
C     III) Print radial distribution function
C
C     Time average radial distribution function
C     RDFs
      do ival=1,id
        nr=shell(ival)
        itype=typ(ival,1)
        jtype=typ(ival,2) 
        rjat(ival)=rjat(ival)/(numb(itype)*nshots)
        gr(ival)=box*box*box*rjat(ival)/(volume(nr)*numb(jtype))
      enddo
      allocate (totnumb(nptypes,nptypes,0:nrmax))
      do itype=1,nptypes
        do jtype=itype,nptypes
          totnumb(itype,jtype,0)=0.0d0
          totnumb(jtype,itype,0)=0.0d0
        enddo
      enddo 
      do nr=1,nrmax
        do itype=1,nptypes
          do jtype=1,nptypes
            ival=pointer(itype,jtype,nr)
            totnumb(itype,jtype,nr)=totnumb(itype,jtype,nr-1)
     $                              +rjat(ival)
          enddo
        enddo
      enddo
C     CM RDFs
      do ival=1,idch
        nr=shch(ival)
        itype=typch(ival,1)
        jtype=typch(ival,2)
        rjch(ival)=rjch(ival)/(numch(itype)*nshots)
        grtot(ival)=box*box*box*rjch(ival)/(volume(nr)*numch(jtype))
      enddo
      allocate (totnumbch(npchs,npchs,0:nrmax))
      do itype=1,npchs
        do jtype=itype,npchs
          totnumbch(itype,jtype,0)=0.0d0
          totnumbch(jtype,itype,0)=0.0d0
        enddo
      enddo
      do nr=1,nrmax
        do itype=1,npchs
          do jtype=1,npchs
            ival=pointch(itype,jtype,nr)
            totnumbch(itype,jtype,nr)=totnumbch(itype,jtype,nr-1)
     $                               +rjch(ival)
          enddo
        enddo
      enddo
C     Open outputfiles
      open (88,file=fname2,status='new')
      open (89,file=fname3,status='new')
C     Writing outputfiles
      write(88,*) 'distance [Angs.]---reference type---type---'
     $          //'---coordinator number---radial dist. function'
      do ival=1,id
        itype=typ(ival,1) 
        jtype=typ(ival,2)
        nr=shell(ival)
        write(88,'(1x,f9.6,1x,a5,1x,a5,1x,e11.4,1x,e11.4)') dist(nr),
     $ atm_name(itype),atm_name(jtype),totnumb(itype,jtype,nr),
     $ gr(ival)
      end do        
      close(88)
      write(89,*) 'distance [Angs.]---reference type---type---'
     $          //'---coordinator number---radial dist. function'
      do ival=1,idch
        itype=typch(ival,1)
        jtype=typch(ival,2)
        nr=shch(ival)
        write(89,'(1x,f9.6,1x,a5,1x,a5,1x,e11.4,1x,e11.4)') dist(nr),
     $       molec_labels(itype),molec_labels(jtype),
     $       totnumbch(itype,jtype,nr),grtot(ival)
      end do
      close(89)
C
C     IV) End of run
C
      write(6,*) 'End of run Ok!!!'
C
      end
