      include "readall.f"
C
C     Real*4 runwrap
C
      SUBROUTINE runwrap(xrf,xunrf)  ! customized
C
      implicit none
      include "params.h"
      include "dimensions.h"
      include "commons.h"
C
C     *****local variables
C
      integer iat,jat,ibond,kk
      real*4 xrf(3,maxat),xunrf(3,maxat),diff
      real*4 rboxgen,el2gen
      common /msd1/ rboxgen,el2gen
C
      do iat = 1,nat
        xunrf(1,iat) = xrf(1,iat)
        xunrf(2,iat) = xrf(2,iat)
        xunrf(3,iat) = xrf(3,iat)
      end do
C
C     *****unwrap by bonds
C
      do ibond = 1,nbonds
        iat = bonds(1,ibond)
        jat = bonds(2,ibond)
        do kk = 1,3
          diff = xrf(kk,jat) - xrf(kk,iat)
          if (abs(diff) .gt. el2gen)
     $      diff = diff - sign(rboxgen,diff)
          xunrf(kk,jat) = xunrf(kk,iat) + diff
        end do
      end do
      return
      end
C
C     ***  Main Radius of gyration program *********
C
C     version 1.0 written by C. J. F. Solano
C
      PROGRAM radius_of_gyration
      implicit none
C
      include "params.h"
      include "dimensions.h"
      include "commons.h"
C
      real*8 qtmp,echain(maxnch),chainmass,rg
      real*8 xcm(3,maxnch),xval(maxnch)
      real*4 xx(3,maxat),xun(3,maxat)
      real*4 time
      real*4 rboxgen,el2gen
      real*8, allocatable :: ech(:),rg2(:)
      integer nfiles,chaintype(maxnch)
      integer iat,itype,ich,ntot,itmp
      integer ifiles,natoms,isnap,i,j,ipos,nshots,iii
      integer, allocatable :: ifn(:),iln(:),numch(:),numchat(:)
      character*50, allocatable :: fname(:)
      common /msd1/ rboxgen,el2gen
C
C     I) Initializations
C
C     Read input.file file
      call readinit()
C     Files for analysis
      open(22,file=fname1,status="old")
      read(22,*) nfiles
      if (nfiles.lt.1) then 
        write(6,*) 'nfiles < 1'
        stop
      endif
      allocate (fname(nfiles))
      do i=1,nfiles
        read(22,'(a50)') fname(i)
        write(6,'(a50)') fname(i)
        open (30+i,file=fname(i),form='unformatted',status='old')
      end do
      write(6,*) 'Opened ',nfiles,' fort.77 (Instantaneous' 
     $         //' Coordinates) files'
      close(22)
C     Read ff.dat, connectivity.dat and coords.inp files  
      call read12() ! q
      call read11() ! nat,chain,natch
      call read26() ! atomtype,box,mass,ldummy
C     Determine possible chain (molecule) types, charges of chains, 
C     and number of chains of each type
      do ich=1,maxnch
        echain(ich)=0.0d0
      end do
      do iat=1,nat
        echain(chain(iat))=echain(chain(iat))+q(atomtype(iat))
      end do  ! next iat
      allocate (ech(npchs),ifn(npchs),iln(npchs),
     $          numch(npchs),numchat(npchs),rg2(npchs))
      ! chain type '1'
      ifn(1)=1
      itype=1
      itmp=natch(1)
      qtmp=echain(1)
      ich=1
      do while (ich.lt.maxnch)
        ich=ich+1
        if (natch(ich).ne.itmp.or.
     $      abs(echain(ich)-qtmp).gt.0.01) then
          ! chain type 'itype'
          iln(itype)=ich-1
          ! chain type 'itype+1'
          itype=itype+1
          if (itype.gt.npchs) then
            write(6,*) 'Number of chain types is greather than ',
     $                 npchs
            stop
          endif
          ifn(itype)=ich
          itmp=natch(ich)
          qtmp=echain(ich)
        endif
      end do  ! next ich
      ! chain type 'itype'
      iln(itype)=maxnch
      if (itype.ne.npchs) then
        write(6,*) 'In automatically assignment there are ',itype,
     $             ' chain types'
        stop  
      endif
      do i=1,npchs
        write(6,*) molec_labels(i),' type for chains from ',ifn(i),
     $             ' to ',iln(i)     
      end do
C
      do i=1,npchs
C       Determine the charge for the first chain of this type
        ech(i)=0.0d0
        numchat(i)=0
        do iat=1,nat
          if (chain(iat).eq.ifn(i)) then  ! choose the first chain of the type
            ech(i)=ech(i)+q(atomtype(iat))
            if (.not.ldummy(iat)) numchat(i)=numchat(i)+1
          endif
        end do   ! next iat
C       Check to make sure that all chain of the same type have the same charge
C       and obtain 'chaintype'
        do j=ifn(i),iln(i)
          if (abs(echain(j)-ech(i)).gt.1d-5)  then
            write(6,*) 'For the chain ',j,' charge = ',echain(j)
            write(6,*) 'but for the chain type ',i,' it is = ',
     $                  ech(i)
            stop
          endif 
          chaintype(j)=i
        end do ! next j
        write(6,*) 'For the chain type ',molec_labels(i),
     $             ' the charge is ',ech(i)
        write(6,*) 'For the chain type ',molec_labels(i),
     $             ' the number of atoms is ',numchat(i)
      end do ! next i
C     Determine number of chains of each type and total number of chains
      ntot=0
      do i=1,npchs
        numch(i)=iln(i)-ifn(i)+1
        write(6,*) 'Type ',molec_labels(i),' has ',
     $              numch(i),' molecules'
        ntot=ntot+numch(i)
      end do
      write(6,*) 'There are ',ntot,' molecules'
C
C     II) Square of radius of gyration
C
      do i=1,npchs
        rg2(i)=0.0d0
      end do
      ipos=0 ! counter for positions along time
      iii=0 ! counter for two-lines block in fort.77 files
      do ifiles = 31,30+nfiles
        ok = .true.
        do while (ok)
C         Read data from fort.77 file
          read(ifiles,iostat=i) natoms,time,isnap,box ! first line 
          if (i.eq.0) then
            if (natoms.ne.nat) then
              write(6,*) 'Wrong number of atoms in fort.77 file'
              write(6,*) natoms,' atoms is different to ',nat
              stop
            endif
            read(ifiles,iostat=i) xx ! second line
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          endif
          if (i.eq.0) then
            ok = .true.
            iii=iii+1
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          else
            ok = .false. ! EOF
          endif
          if (ok .and. iii.gt.nskip) then 
            ipos=ipos+1
            if (mod(ipos,nread).eq.0) then
              nshots=ipos/nread
C             Periodic cubic cell
              rboxgen=real(box)
              el2gen=0.5*rboxgen
              call runwrap(xx,xun)
C             Obtention of center of mass for each chain in a given time 
              do ich=1,maxnch
                do j=1,3
                  xcm(j,ich)=0.0d0 ! time-dependent coordinates for the center of mass
                end do
                chainmass=0.0d0
                do iat=1,nat
                  if (chain(iat).eq.ich) then
                    chainmass=chainmass+mass(iat)
                    do j=1,3
                      xcm(j,ich)=xcm(j,ich)+xun(j,iat)*mass(iat)
                    end do
                  endif
                end do
                do j=1,3
                  xcm(j,ich)=xcm(j,ich)/chainmass
                end do
                xval(ich)=0.0d0
                do iat=1,nat
                  if (chain(iat).eq.ich) then
                    do j=1,3
                      qtmp=dble(xun(j,iat))-xcm(j,ich)
                      xval(ich)=xval(ich)+qtmp*qtmp
                    end do
                  end if
                end do
              end do ! next ich
C             Square of radius of gyration
              do i=1,npchs
                do j=ifn(i),iln(i)
                  rg2(i)=rg2(i)+xval(j)
                end do
              end do
            endif ! (mod(ipos,nread).eq.0)
          endif ! ok .and. iii.gt.nskip
        enddo
        write(6,*) 'fort.77 file ',ifiles,':'
        write(6,*) natoms,' atoms, ',time,' time step (fs), ',box,
     $             ' cell size (Angs.)' 
        write(6,*) isnap,' final snapshot file, ',ipos,
     $             ' actual snapshot'
        close(ifiles)
      end do ! next ifiles
C      
C     III) Print radius of gyration
C
C     Open outputfiles
      open (88,file=fname2,status='new')
C     Writing outputfiles
      write(88,'(a)') '# ****************** #'
      write(88,'(a)') '# Radius of gyration #'
      write(88,'(a)') '# ****************** #'
      write(88,'(a)') '# Symbols and units : '
      write(88,'(a)') '# rg2 -> Square of radius of gyration (Angs**2)'
      write(88,'(a)') '# rg  -> Radius of gyration (Angs)'
      write(88,'(a)') '# Chain type---rg2---rg'
      do i=1,npchs
        rg2(i)=rg2(i)/(nshots*numch(i)*numchat(i))
        rg=dsqrt(rg2(i))
        write(88,'(1x,a,1x,e14.6,1x,e14,6)') molec_labels(i),rg2(i),rg
      end do
      close(88)
C
C     IV) End of run
C
      write(6,*) 'End of run Ok!!!'
C
      end
