C
C     ***  Thermodynamics properties program ***
C
C     version v1.0 written by C. J. F. Solano
C
      PROGRAM thermodynamics
      implicit none
C *** LOCAL VARIABLES
      character*144 line
      character*50 fname
      integer      nskip,nread,i,ntot
      real*8, allocatable :: time(:),temper(:),press(:),box(:),E(:),H(:)
      real*8       step,stepold,var
      logical      Qtemp,Qpress,Qbox,QE,QH
      real*8       avtemp,avpress,avbox,avE
      real*8       stdtemp,stdpress,stdbox,stdE
C *** READ input.file    
      open(25,file="input.file",status="old")
      write(6,*) '**** READING input.file FILE ***'
      CALL read_a_line(25,line)
      read (line,*) fname
      write(6,*) fname,' fort.65 file'
      CALL read_a_line(25,line)
      read (line,*) nskip,nread
      if (nread.lt.1) then
        write(6,*) 'nread < 1'
        stop
      endif
      if (nskip.lt.0) then
        write(6,*) 'nskip < 0'
        stop
      endif
      if (nskip.ge.nread) then
        write(6,*) 'nskip => nread'
        stop
      end if
      write(6,*) '(nskip,nread)=(',nskip,',',nread,')'
      CALL read_a_line(25,line)
      read (line,*) Qtemp,Qpress,Qbox,QE,QH
      if (QH) write(6,*) 'Hamiltonian conservation calculation'
      if (QE) write(6,*) 'Average energy calculation'
      if (Qtemp) write(6,*) 'Average temperature calculation'
      if (Qpress) write(6,*) 'Average pressure calculation'
      if (Qbox) write(6,*) 'Average box calculation'
      close(25)
C *** READ DATA file and OBTENTION of thermodynamic poperties
      open(25,file=fname,status="old")
      do i=1,nskip
        read(25,*) line
      enddo
      ntot = nread-nskip
      allocate (time(ntot),temper(ntot),press(ntot),box(ntot),
     $          E(ntot),H(ntot))
      read(25,*) time(1),temper(1),press(1),box(1),E(1),H(1)
      if (QH) var = 0.0d0
      if (Qtemp) avtemp = temper(1)
      if (Qpress) avpress = press(1)
      if (Qbox) avbox = box(1)
      if (QE) avE = E(1)
      do i=2,ntot
        read(25,*) time(i),temper(i),press(i),box(i),E(i),H(i)
        if (QH) var = var + dabs(H(i)-H(1))
        if (Qtemp) avtemp = avtemp + temper(i)
        if (Qpress) avpress = avpress + press(i)
        if (Qbox) avbox = avbox + box(i)
        if (QE) avE = avE + E(i)
        step = time(i) - time(i-1)
        if (i.eq.2) stepold = step
        if (step.ne.stepold) then
          write(6,*) 'Two different time steps (fs) :',step,stepold
          stop
        end if  
      end do
      if (QH) var = var/((ntot-1.0d0)*dabs(H(1)))
      if (Qtemp) then 
        avtemp = avtemp/ntot
        stdtemp = 0.0d0
        do i=1,ntot
          stdtemp = stdtemp + (temper(i)-avtemp)*(temper(i)-avtemp)
        end do
        stdtemp = dsqrt(stdtemp/ntot)
      end if
      if (Qpress) then
        avpress = avpress/ntot
        stdpress = 0.0d0
        do i=1,ntot
          stdpress = stdpress + (press(i)-avpress)*(press(i)-avpress)
        end do
        stdpress = dsqrt(stdpress/ntot)
      end if
      if (Qbox) then 
        avbox = avbox/ntot
        stdbox = 0.0d0
        do i=1,ntot
          stdbox = stdbox + (box(i)-avbox)*(box(i)-avbox)
        end do
        stdbox = dsqrt(stdbox/ntot)
      end if
      if (QE) then 
        avE = avE/ntot
        stdE = 0.0d0
        do i=1,ntot
          stdE = stdE + (E(i)-avE)*(E(i)-avE)
        end do
        stdE =  dsqrt(stdE/ntot)
      end if
      write(6,*) 'timestep (fs) = ',step
      if (QH) then
        write(6,*) '**** HALMILTONIAN CONSERVATION ****'
        write(6,*) 'Hamiltonian conservation H, log(H) = ',var,log(var)
      end if
      if (QE) then
        write(6,*) '**** AVERAGE ENERGY ****'
        write(6,*) 'E (Kcal/mol) = ',avE
        write(6,*) 'Std. dev. (Kcal/mol) = ',stdE
      end if
      if (Qtemp) then
        write(6,*) '**** AVERAGE TEMPERATURE ****'
        write(6,*) 'T (K) = ',avtemp
        write(6,*) 'Std. dev. (K) = ',stdtemp
      end if
      if (Qpress) then
        write(6,*) '**** AVERAGE PRESSURE ****'
        write(6,*) 'P (atm) = ',avpress
        write(6,*) 'Std. dev. (atm) = ',stdpress
      end if
      if (Qbox) then
        write(6,*) '**** AVERAGE BOX ****'
        write(6,*) 'box (Angstrom) = ',avbox
        write(6,*) 'Std. dev. (Angstrom) = ',stdbox
      end if 
C *** END
      write(6,*) 'End of run Ok!!!'
      end
C
      SUBROUTINE read_a_line(iunit,line)
C
      implicit none
C
C     *****shared variables
C
      integer iunit
      character*144 line
C
1     read(iunit,'(a144)',end=100) line
      if (line(1:1) .eq. '*') goto 1
      return
100   continue
      write (6,*) 'end of file ',iunit,' was reached'
      stop
      end
