      PROGRAM new_visc
C
C     version v1.2 written by C. J. F. Solano
C     Simpson's rule is used for integration
      implicit none
      integer*8 maxshots
      parameter (maxshots=12000000)
C     LOCAL VARIABLES
      real*4 time,shstress(6) ! fort.78 files 
      real*4 Ptrace,Ptress(6,maxshots)
      real*8 temper,box,volume,prefact,pressure,afact,ttime,visc
      real*8 sum(6)
      integer nfiles,nread,nskip,nshots,norigin,initval,nposfq
      integer i,ipos,iii,ifiles,iflag,npos,ik,itek,k,j
      logical ok,borodin,stfile
      character*50 fname1
      character*50, allocatable :: fname(:)
C
C     I) Initializations
C
C     Files for analysis
      write(6,*) 'Enter name as fname up to 12 char'
      write(6,*) 'fname - names of fort.78 files'
      read(5,*) fname1
      open(22,file=fname1,status='old')
      read(22,*) nfiles
      allocate (fname(nfiles))
      do i=1,nfiles
        read(22,'(a50)') fname(i)
        write(6,'(a50)') fname(i)
        open (30+i,file=fname(i),form='unformatted',status='old')
      end do
      write(6,*) 'Opened ',nfiles,' fort.78 (Pressure Tensor) files'
      close(22)
C
      write(6,*) 'How often to read snapshots'
      read(5,*) nread
      write(6,*) 'How many to skip in fort.78 files'
      read(5,*) nskip
      write(6,*) 'Enter temperature [in K]'
      read(5,*) temper
      write(6,*) 'Enter cubic box lentgh [in Angstroms]'
      read(5,*) box
      volume=box*box*box
      write(6,*) 'Simulation cell volume [in Angstroms] = ',volume
      stfile=.false.
      write(6,*) 'Do you want to print symmetrized traceless portion'
      write(6,*) 'of the stress tensor? (1-yes,0-no)'
      read(5,*) iflag
      if (iflag.eq.1) then
        write(6,*) 'Enter name as fname up to 12 char'
        write(6,*) 'fname - names of symmetrized traceless portion'
        write(6,*) '        of the stress tensor file'
        read(5,*) fname1
        open (78,file=fname1,status='new')
        stfile=.true.
      endif
      write(6,*) 'Enter name as fname up to 12 char'
      write(6,*) 'fname - shear viscosity'
      read(5,*) fname1
      borodin=.false.
      write(6,*) 'Do you want to use Borodin algorithm?'
     $         //' (1-yes,0-no)'
      read(5,*) iflag
      if (iflag.eq.1) borodin=.true.
C     
C     II) Symmetrized traceless portion of the stress tensor
C
      ipos=0 ! counter for stress tensor along time
      iii=0 ! counter for two-lines block in fort.78 files
      pressure=0.0d0
      if (borodin) then
        do i=1,6
          sum(i)=0.0d0
        enddo
      endif
      if (stfile) then
        do i=1,nfiles
          write(78,'("# fort.78 file = ",a50)') fname(i)
        enddo
        write(78,'(a,i10)') "# v1.0:nfiles ",nfiles
        write(78,'(a)') "# time -> elapsed time [ps]"
        write(78,'(a)') "# Pab; a,b=x,y,z -> symmetrized traceless"
        write(78,'(a)') "#      portion of the stress tensor [atm]"
        write(78,'(a)') "# time---Pxy---Pxz---Pyz---Pxx---Pyy---Pzz"
      endif
      do ifiles=31,30+nfiles
        ok=.true.
        do while (ok)
C         Read data from fort.78 file
          read(ifiles,iostat=i) time ! first line
          if (i.eq.0) then
            read(ifiles,iostat=i) shstress ! second line
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          endif
          if (i.eq.0) then
            ok = .true.
            iii=iii+1
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          else
            ok = .false. ! EOF
          endif
          if (ok) then
            if (iii.gt.nskip) ipos=ipos+1
C           Obtention of symmetrized traceless portion of the stress tensor
            if (ipos.gt.0 .and. mod(ipos,nread).eq.0) then
              nshots=ipos/nread
              if (maxshots.lt.nshots) then
                write(6,*) 'Increase the maxshots'
                write(6,*) 'Current maxshots = ',maxshots
                stop
              endif
              Ptrace=(shstress(1)+shstress(2)+shstress(3))/3.0
              Ptress(1,nshots)=shstress(4) !Pxy(t)
              Ptress(2,nshots)=shstress(5) !Pxz(t)  
              Ptress(3,nshots)=shstress(6) !Pyz(t)  
              Ptress(4,nshots)=shstress(1)-Ptrace ! Pxx(t)
              Ptress(5,nshots)=shstress(2)-Ptrace ! Pyy(t) 
              Ptress(6,nshots)=shstress(3)-Ptrace ! Pzz(t)   
              pressure=pressure+Ptrace
              if (borodin) then
                do i=1,6
                  sum(i)=sum(i)+Ptress(i,nshots)
                enddo
              endif
              if (stfile .and. .not.borodin) then
                write(78,'(F10.3,6E15.5)') dble(ipos)*time/1000.0d0,
     $  Ptress(1,nshots),Ptress(2,nshots),Ptress(3,nshots),
     $  Ptress(4,nshots),Ptress(5,nshots),Ptress(6,nshots)
              endif
            endif ! ipos.gt.0 .and. mod(ipos,nread).eq.0
          endif ! ok
        enddo
        write(6,*) 'fort.78 file ',ifiles,':',ipos,' actual snapshot'
        close(ifiles)
      enddo ! next ifiles
      if (nshots.eq.0) then
        write(6,*) 'Number of snaphost is equal to 0'
        stop
      endif
      pressure=pressure/dble(nshots)
      write(6,*) 'Virial theorem. Pressure (atm) = ',pressure
      if (borodin) then
C       Time average of symmetrized traceless portion of stress tensor
        do i=1,6
          sum(i)=sum(i)/dble(nshots)
          write(6,*) 'Time average of Pab:i--<Pab(i)>,',i,sum(i)
        enddo
C       Desviation of symmetrized traceless portion of stress tensor
        do i=1,nshots
          do k=1,6
            Ptress(k,i)=Ptress(k,i)-sum(k)
          enddo
          if (stfile) then
             write(78,'(F10.3,6E15.5)')
     $                dble(i)*dble(nread)*time/1000.0d0,
     $                Ptress(1,i),Ptress(2,i),Ptress(3,i),
     $                Ptress(4,i),Ptress(5,i),Ptress(6,i)
          endif
        enddo
      endif
      if (stfile) close (78)
C
C     III) Constant factor
C
      prefact=volume*101325.0d0**2*dble(nread)*time/
     &        (1.0d23*1.380650424*temper)
C
C     IV) Shear viscosity
C      
C     Obtention of time origins
      norigin=nshots/2
      if (norigin.lt.4) then
        write(6,*) norigin,' time origins is lower than 4'
        stop
      endif
C     Open outputfiles
      open (79,file=fname1,status='new')
      do i=1,nfiles
        write(79,'("# fort.78 file = ",a50)') fname(i)
      enddo ! next i
      write(79,'(a,i10)') "# v1.2:nfiles ",nfiles
      write(79,'(a)') "# time -> elapsed time [ps]"
      write(79,'(a)') "# visc -> shear viscosity [Pa s]"
      write(79,'(a)') "# time---visc"
C     Initial elapsed time
      initval=2
      write(6,*) 'Do you want to change initial elapsed time?'
     $         //' (1-yes,0-no)'
      read(5,*) iflag
      if (iflag.eq.1) then
        write(6,*) 'Enter initial elapsed time value'
        read(5,*) initval
        if (initval.lt.2) then
          write(6,*) 'Initial elapsed time value is lower than 2'
          stop
        endif
        if (initval.gt.norigin) then
          write(6,*) 'Initial elapsed time value is greather than'
     $             //' maximum value of ',norigin
          stop
        endif
        if (mod(initval,2).ne.0) then
          write(6,*) '****warning****'
          write(6,*) 'Initial elapsed time value is an odd number'
          write(6,*) initval,' initial value'
          initval=initval-1
          write(6,*) initval,' actual value'
        endif
      endif
      nposfq=2
      write(6,*) 'Do you want to change freq. in the elapsed time?'
     $         //' (1-yes,0-no)'
      read(5,*) iflag
      if (iflag.eq.1) then
        write(6,*) 'Enter freq. in the elapsed time'
        read(5,*) nposfq
        if (nposfq.lt.2) then
          write(6,*) 'Freq. in the elapsed time is lower than 2'
          stop
        endif
        if (nposfq.gt.(norigin-initval)) then
          write(6,*) 'Freq. in the elapsed time is greather than'
     $             //' maximum value of ',norigin-initval
          stop
        endif
        if (mod(nposfq,2).ne.0) then
          write(6,*) '****warning****'
          write(6,*) 'Freq. in the elapsed time is an odd number'
          write(6,*) nposfq,' initial value'
          nposfq=nposfq-1
          write(6,*) nposfq,' actual value'
        endif
      endif      
C
      do npos=initval,norigin,nposfq ! npos is the elapsed time
        visc=0.0d0
        do ik=1,norigin !ik is the starting position
          itek=ik+npos ! itek is the final position
          do k=1,6 
C        ***************************
C        * Extended Simpson's rule *
C        ***************************
            afact=dble(Ptress(k,ik)+Ptress(k,itek))/3.0d0
            j=0
            do i=ik+1,itek-1
              j=j+1
              if (mod(j,2).eq.0) then
                afact=afact+2.0d0*dble(Ptress(k,i))/3.0d0
              else
                afact=afact+4.0d0*dble(Ptress(k,i))/3.0d0
              endif
            enddo ! next i
C         **************************
            if (k.le.3) then ! xy,xz,yz
              visc=visc+afact*afact
            else ! xx,yy,zz
              visc=visc+2.0d0*afact*afact/3.0d0
            endif
          enddo ! next k
        enddo ! next ik
c       elapsed time [in ps]
        ttime=dble(npos)*dble(nread)*time/1000.0d0
c       write outputfile
        visc=prefact*visc/(dble(norigin)*dble(npos))
        write(79,'(F10.3,E15.5)') ttime,visc
      enddo ! next npos
      close(79)
C
C     V) End of run
C
      write(6,*) 'End of run Ok!!!'
C
      end

