      PROGRAM new_visc
C
C     version v1.3 written by C. J. F. Solano
C     Simpson's rule is used for integration
C
      implicit none
      integer*8 maxshots
      parameter (maxshots=12000000)
C     LOCAL VARIABLES
      real*8 EPS
      real*4 time,shstress(6) ! fort.78 files 
      real*4 Ptrace,Ptress(6,maxshots)
      real*8 temper,box,volume,prefact,pressure,afact,ttime,visc
      real*8 sum2(6)
      real*8 sum,os,ost,s
      integer nfiles,nread,nskip,nshots,norigin,initval,nposfq
      integer i,ipos,iii,ifiles,iflag,npos,ik,itek,k
      integer j,l,m,it,nval,ncalls
      logical ok,conv,borodin,stfile
      character*50 fname1
      character*50, allocatable :: fname(:)
C
C     I) Initializations
C
C     Files for analysis
      write(6,*) 'Enter name as fname up to 12 char'
      read(5,*) fname1
      open(22,file=fname1,status='old')
      read(22,*) nfiles
      allocate (fname(nfiles))
      do i=1,nfiles
        read(22,'(a50)') fname(i)
        write(6,'(a50)') fname(i)
        open (30+i,file=fname(i),form='unformatted',status='old')
      end do
      write(6,*) 'Opened ',nfiles,' fort.78 (Pressure Tensor) files'
      close(22)
C
      write(6,*) 'How often to read snapshots'
      read(5,*) nread
      write(6,*) 'How many to skip in fort.78 files'
      read(5,*) nskip
      write(6,*) 'Enter temperature [in K]'
      read(5,*) temper
      write(6,*) 'Enter cubic box lentgh [in Angstroms]'
      read(5,*) box
      volume=box*box*box
      write(6,*) 'Simulation cell volume [in Angstroms] = ',volume
      stfile=.false.
      write(6,*) 'Do you want to print symmetrized traceless portion'
      write(6,*) 'of the stress tensor? (1-yes,0-no)'
      read(5,*) iflag
      if (iflag.eq.1) then
        write(6,*) 'Enter name as fname up to 12 char'
        write(6,*) 'fname - names of symmetrized traceless portion'
        write(6,*) '        of the stress tensor file'
        read(5,*) fname1
        open (78,file=fname1,status='new')
        stfile=.true.
      endif
      write(6,*) 'Enter name as fname up to 12 char'
      write(6,*) 'fname - shear viscosity'
      read(5,*) fname1
      write(6,*) 'Enter fractional accuracy'
      read(5,*) EPS
      if (EPS.lt.0.0d0) then
        write(6,*) 'Fractional accuracy is lower than 0'
        stop
      endif
      write(6,*) 'Fractional accuracy = ',EPS
      borodin=.false.
      write(6,*) 'Do you want to use Borodin algorithm?'
     $         //' (1-yes,0-no)'
      read(5,*) iflag
      if (iflag.eq.1) borodin=.true.
C     
C     II) Symmetrized traceless portion of the stress tensor
C
      ipos=0 ! counter for stress tensor along time
      iii=0 ! counter for two-lines block in fort.78 files
      pressure=0.0d0
      if (borodin) then
        do i=1,6
          sum2(i)=0.0d0
        enddo
      endif
      if (stfile) then
        do i=1,nfiles
          write(78,'("# fort.78 file = ",a50)') fname(i)
        enddo
        write(78,'(a,i10)') "# v1.0:nfiles ",nfiles
        write(78,'(a)') "# time -> elapsed time [ps]"
        write(78,'(a)') "# Pab; a,b=x,y,z -> symmetrized traceless"
        write(78,'(a)') "#      portion of the stress tensor [atm]"
        write(78,'(a)') "# time---Pxy---Pxz---Pyz---Pxx---Pyy---Pzz"
      endif
      do ifiles=31,30+nfiles
        ok=.true.
        do while (ok)
C         Read data from fort.78 file
          read(ifiles,iostat=i) time ! first line
          if (i.eq.0) then
            read(ifiles,iostat=i) shstress ! second line
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          endif
          if (i.eq.0) then
            ok=.true.
            iii=iii+1
          else if (i.gt.0) then
            write(6,*) 'Error during read execution'
            stop
          else
            ok=.false. ! EOF
          endif
          if (ok) then
            if (iii.gt.nskip) ipos=ipos+1
C           Obtention of symmetrized traceless portion of the stress tensor
            if (ipos.gt.0 .and. mod(ipos,nread).eq.0) then
              nshots=ipos/nread
              if (maxshots.lt.nshots) then
                write(6,*) 'Increase the maxshots'
                write(6,*) 'Current maxshots = ',maxshots
                stop
              endif
              Ptrace=(shstress(1)+shstress(2)+shstress(3))/3.0
              Ptress(1,nshots)=shstress(4) !Pxy(t)
              Ptress(2,nshots)=shstress(5) !Pxz(t)  
              Ptress(3,nshots)=shstress(6) !Pyz(t)  
              Ptress(4,nshots)=shstress(1)-Ptrace ! Pxx(t)
              Ptress(5,nshots)=shstress(2)-Ptrace ! Pyy(t) 
              Ptress(6,nshots)=shstress(3)-Ptrace ! Pzz(t)   
              pressure=pressure+Ptrace
              if (borodin) then
                do i=1,6
                  sum2(i)=sum2(i)+Ptress(i,nshots)
                enddo
              endif
              if (stfile .and. .not.borodin) then
                write(78,'(F10.3,6E15.5)') dble(ipos)*time/1000.0d0,
     $  Ptress(1,nshots),Ptress(2,nshots),Ptress(3,nshots),
     $  Ptress(4,nshots),Ptress(5,nshots),Ptress(6,nshots)
              endif
            endif ! ipos.gt.0 .and. mod(ipos,nread).eq.0
          endif ! ok
        enddo
        write(6,*) 'fort.78 file ',ifiles,':',ipos,' actual snapshot'
        close(ifiles)
      enddo ! next ifiles
      if (nshots.eq.0) then
        write(6,*) 'Number of snaphost is equal to 0'
        stop
      endif
      pressure=pressure/dble(nshots)
      write(6,*) 'Virial theorem. Pressure (atm) = ',pressure
      if (borodin) then
C       Time average of symmetrized traceless portion of stress tensor
        do i=1,6
          sum2(i)=sum2(i)/dble(nshots)
          write(6,*) 'Time average of Pab:i--<Pab(i)>,',i,sum2(i)
        enddo
C       Desviation of symmetrized traceless portion of stress tensor
        do i=1,nshots
          do k=1,6
            Ptress(k,i)=Ptress(k,i)-sum2(k)
          enddo
          if (stfile) then
             write(78,'(F10.3,6E15.5)')
     $                dble(i)*dble(nread)*time/1000.0d0,
     $                Ptress(1,i),Ptress(2,i),Ptress(3,i),
     $                Ptress(4,i),Ptress(5,i),Ptress(6,i)
          endif
        enddo
      endif
      if (stfile) close (78)
C
C     III) Constant factor
C
      prefact=volume*101325.0d0**2*dble(nread)*time/
     &        (1.0d23*1.380650424*temper)
C
C     IV) Shear viscosity
C      
C     Obtention of time origins
      norigin=nshots/2
      if (norigin.lt.1024) then
        write(6,*) norigin,' time origins is lower than 1024'
        stop
      endif  
C     Open outputfiles
      open (79,file=fname1,status='new')
      do i=1,nfiles
        write(79,'("# fort.78 file = ",a50)') fname(i)
      enddo ! next i
      write(79,'(a,i10)') "# v1.3:nfiles ",nfiles
      write(79,'(a)') "# time -> elapsed time [ps]"
      write(79,'(a)') "# visc -> shear viscosity [Pa s]"
      write(79,'(a)') "# time---visc"
C
      npos=1024 ! initial elapsed time
      ncalls=11 ! initial number of steps for trapzd routine 
      do while (npos.le.norigin)
        write(6,*) npos,' elapsed time' 
        write(6,*) ncalls,' number of calls'    
        visc=0.0d0
        conv=.true.
        ik=1 ! initial value for starting position
        itek=ik+npos ! initial value for final position
        do while (ik.le.norigin .and. conv)
          k=0
          do while (k.lt.6 .and. conv)
            k=k+1
C           +++++++++++++++++++++++++++++++
C           + Adaptation of qsimp routine +
C           +  ( NUMERICAL RECIPES IN     +
C           +       FORTRAN 77        )   +
C           +++++++++++++++++++++++++++++++    
            ost=-1.0d30
            os=-1.0d30
            ok=.true.
            j=0 
            do while (j.lt.ncalls .and. ok)
              j=j+1
C             ********************************
C             * Adaptation of trapzd routine *
C             *  ( NUMERICAL RECIPES IN      *
C             *       FORTRAN 77        )    *
C             ********************************
              if (j.eq.1) then
                afact=0.5d0*dble(npos)*dble(Ptress(k,ik)
     $                                     +Ptress(k,itek))
              else
                it=2**(j-2)
                nval=npos/it ! spacing of the points to be added
                l=ik+nval/2 
                sum=0.0d0
                do m=1,it
                  sum=sum+Ptress(k,l)
                  l=l+nval
                enddo   
                afact=0.5d0*(afact+dble(nval)*sum) ! replaces afact by its refined value
              endif
C             ***************************** 
              s=(4.0d0*afact-ost)/3.0d0
              if (j.gt.5) then ! avoid spurious early convergence
                if (abs(s-os).lt.EPS*abs(os) .or.
     $              (s.eq.0.0d0.and.os.eq.0.0d0)) ok=.false.
              endif
              if (j.eq.ncalls .and. ok) then
                conv=.false.
                write(6,*) 'Integral does not converge'
                write(6,*) 'Kval = ',k
                write(6,*) 'Origin point = ',ik
                write(6,*) 'Final point  = ',itek
              endif
              if (ok) then 
                os=s
                ost=afact
              endif 
            enddo 
C           +++++++++++++++++++++++++++++++       
            if (conv) then     
              if (k.le.3) then ! xy,xz,yz
                visc=visc+afact*afact
              else ! xx,yy,zz
                visc=visc+2.0d0*afact*afact/3.0d0
              endif
            endif 
          enddo ! next k
          if (conv) then
            ik=ik+1
            itek=ik+npos
          endif
        enddo ! next ik
        if (conv) then
c         elapsed time [in ps]
          ttime=dble(npos)*dble(nread)*time/1000.0d0
c         write outputfile
          visc=prefact*visc/(dble(norigin)*dble(npos))
          write(79,'(F10.3,E15.5)') ttime,visc
        endif 
        ncalls=ncalls+1
        npos=npos*2
      enddo ! next npos
      close(79)
C
C     V) End of run
C
      write(6,*) 'End of run Ok!!!'
C
      end
